/*
 * CMovieTDIF.hpp
 * Copyright (C) 2010 Andreas Fiedler <smartsmurf@users.sourceforge.net>
 *
 * This file is part of samygodca, a free DTS Coherent Acoustics
 * stream decoder plugin for SamyGO extension of Samsung DTV.
 *
 * samygodca is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * samygodca is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef __CMOVIETDIF_H__
#define __CMOVIETDIF_H__

#include "Config.h"
#include "PCSemaphore.hpp"
#include "PCTask.hpp"
#include "Multimedia.hpp"


// class LIB_EXPORT CMovieTDIF
class LIB_EXPORT CMovieTDIF : protected PCTask
{
private:
	int	mRes0000[0x0566];
	PCSemaphore Sem1;
	int	mRes0001;
	// PCTask m_PCTask;

public:
	struct _tSetAudioAAC {
		unsigned char type;
		unsigned char samplerate_index;
		unsigned char sub_id;
		unsigned char channels;
		int sample_rate;
		int e8;
		unsigned char e12;
		unsigned char e13;
		unsigned char e14;
		unsigned char e15;
		unsigned char e16;
		unsigned char e17;
		unsigned char e18;
	};

	struct _tSetAudioPCM {
		int eFsRate;
		int bitWidth;
		int channels;
		int blockAlign;
		int ePCM;
		int eADPCM;
		int bLittleEndian;
	};

	struct _tSetAudioWMA {
		short codec_tag;
		short channels;
		int sample_rate;
		int byte_rate;
		int block_align;
		short bits_per_sample;
	};

	struct MovieSettings {
		SdContainerFormat_k ContainerFormat_TD;
		SdVideoFormat_k VideoFormat_TD;
		SdAudioFormat_k AudioFormat_TD;
		_tSetAudioPCM eSettingsPCM;
		_tSetAudioAAC eSettingsAAC;
		_tSetAudioWMA eSettingsWMA;
		int	mRes0B8;
		int mRes0BC;
		int64_t Filesize;
		int Duration;
		int	FrameWidth;
		int FrameHeight;
		int mRes0D4;
		float FrameRate;
		int mRes0DC;
		int SampleRate_TD;
		int mRes0E4;
	};

	struct _PositionInfo {
		int Position;
		int Duration;	
	};

	struct _ErrorInfo {
		int ContainerErr;
		int VideoErr;
		int AudioErr;
		int hResolution;
		int vResolution;
	};

	virtual ~CMovieTDIF( void );
//	virtual int t_Main(void);
	int CanTrick(void);
	int CanSeek(void);
    int SetAudioStreamID(unsigned int, CMovieTDIF::MovieSettings);
	int MovePosition(unsigned int, unsigned long long *, bool);
	int Play(bool, bool, bool);
	int GetCurrentTime(void);
	CMovieTDIF( void );
	int Stop(bool,bool);
};


#endif

