/*
 * CParsingMedia.cpp
 * Copyright (C) 2010 Andreas Fiedler <smartsmurf@users.sourceforge.net>
 *
 * This file is part of samygodca, a free DTS Coherent Acoustics
 * stream decoder plugin for SamyGO extension of Samsung DTV.
 *
 * samygodca is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * samygodca is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <dlfcn.h>

#include "CParsingMedia.hpp"
#include "Multimedia.hpp"
#include "Logging.h"
#include "ffmpeg_matroska.h"
#include "ffmpeg_mpegts.h"

DLL_LOCAL av_register_all_t av_register_all222;
DLL_LOCAL av_open_input_file_t av_open_input_file222;
DLL_LOCAL av_close_input_file_t av_close_input_file222;
DLL_LOCAL av_find_stream_info_t av_find_stream_info222;
DLL_LOCAL dump_format_t dump_format222;
DLL_LOCAL getMpeg4AACObjetType_t getMpeg4AACObjetType222;
DLL_LOCAL getMpeg4AACSBR_t getMpeg4AACSBR222;

void * CParsingMedia::hLib_avutil;
void * CParsingMedia::hLib_avcodec;
void * CParsingMedia::hLib_avformat;
DLL_LOCAL void * hLib_avcore;
char * CParsingMedia::dlError;
int CParsingMedia::m_bFirst;

DLL_LOCAL char path_ext_ffmpeg[256];

	/* ffmpeg 0.6 has avformat version 52.77.x 
	   ffmpeg 0.4 (original Samsung B650 TV) has avformat version 52.23.x
	   The matroska file format changed inbetween
	*/
#if LIBAVFORMAT_VERSION_INT < ((52 << 16) | ( 77 << 8))	
 #define S_MatroskaTrack MatroskaTrackOld
 #define S_MatroskaDemuxContextOld MatroskaDemuxContextOld
#else
 #define S_MatroskaTrack MatroskaTrack
 #define S_MatroskaDemuxContext MatroskaDemuxContext
#endif

void CParsingMedia::SetAudioStreamInfo(AVFormatContext * ffMediaContext, unsigned int astream_index, Multimedia::_tMediaInfoContext * media_info_context)
{
	int flag_mkv_audio;

	S_MatroskaTrack		* mkv_tracklist;
	S_MatroskaDemuxContext * mkv_demux_context;

	ROSE_PRINT(0x4,"Processing [%i th] Audio Stream.", astream_index );
	AVStream * current_avstream = ffMediaContext->streams[astream_index];
	ROSE_PRINT(0x4,"[Audio] Stream ID = %i.", current_avstream->id );
	ROSE_PRINT(0x4,"[Audio] Stream index = %i.", current_avstream->index );

	Multimedia::_tStreamInfoContext * current_stream_info_context = &media_info_context->stream_info[astream_index];
	current_stream_info_context->stream_type = Multimedia::STREAM_TYPE_AUDIO;					// offset 0x0238
	flag_mkv_audio = 0;
	mkv_tracklist = NULL;
	mkv_demux_context = NULL;
	current_stream_info_context->stream_id = current_avstream->id;	// offset 0x023C
	if( media_info_context->container_info.nId == Multimedia::CONTAINER_MKV ){
		// 00FBA7C8
		ROSE_PRINT(0x4,"MKV DEMUX Context");
		if( ffMediaContext->priv_data ){
			ROSE_PRINT(0x4,"MKV has priv_data.");
			mkv_demux_context = (S_MatroskaDemuxContext *)ffMediaContext->priv_data;
			ROSE_PRINT(0x4,"MKV number of elements = %i.", mkv_demux_context->tracks.nb_elem );
			ROSE_PRINT(0x4,"sizeof(MatroskaTrack) = %i", sizeof(S_MatroskaTrack) );
			mkv_tracklist = (S_MatroskaTrack *)mkv_demux_context->tracks.elem; 
#if 0
			/* original Samsung code */
			/* BUG: nPid of all streams is min(index of audio streams) */
			for(int i = 0; i < mkv_demux_context->tracks.nb_elem; i++ ){
				if( mkv_tracklist[i].type == MATROSKA_TRACK_TYPE_AUDIO ){
					current_stream_info_context->audio_info.nPid = (int)mkv_tracklist[i].num;
					break;
				}
			}
#else
			/* fixed code to properly assign nPid */
			for(int i = 0; i < mkv_demux_context->tracks.nb_elem; i++ ){
				if( mkv_tracklist[i].type != MATROSKA_TRACK_TYPE_AUDIO )
					continue;
				if( (unsigned)i >= astream_index ){
					current_stream_info_context->audio_info.nPid = (int)mkv_tracklist[i].num;
					break;
				}
			}
#endif
			flag_mkv_audio = 1;
		} else {
			flag_mkv_audio = 0;
			mkv_tracklist = NULL;
			mkv_demux_context = NULL;
		}
	} else {
		// 00FBA2EC
		current_stream_info_context->audio_info.nPid = current_avstream->id;
	}
	ROSE_PRINT(0x4,"pMediaInfo->tStreams[index].tAudioInfo.nPid = %d", current_stream_info_context->audio_info.nPid );
	ROSE_PRINT(0x4,"ffmpeg Audio codec ID = %d", current_avstream->codec->codec_id);

	// PRINTF("codec->sub_id = %i, codec->codec_id = %i", current_avstream->codec->sub_id, current_avstream->codec->codec_id);

	current_stream_info_context->audio_info.nSubId = current_avstream->codec->sub_id;
	switch( current_avstream->codec->codec_id ){
	case CODEC_ID_NONE:
		if( (media_info_context->container_info.nId == Multimedia::CONTAINER_MKV) &&
			(flag_mkv_audio) ){
			ROSE_PRINT(0x4,"Processing MKV...");
			for(int i = 0; i < mkv_demux_context->tracks.nb_elem; i++ ){
				if( mkv_tracklist[i].type == MATROSKA_TRACK_TYPE_AUDIO ){
					if( strcmp(mkv_tracklist[i].codec_id, "A_MS/ACM") == 0 ){
						ROSE_PRINT(0x4,"Format: PCM , Codec Info : A_MS/ACM (MU-LAW)");
						ROSE_PRINT(0x4,"pContext->streams[index]->codec->codec_tag = 0x%x", current_avstream->codec->codec_tag );
						current_avstream->codec->codec_id = CODEC_ID_PCM_MULAW;
						current_stream_info_context->audio_info.AudioCodecID = Multimedia::AUDIO_XLAW; // 0x09
						break;
					}
				}
			}
		} else {
			current_stream_info_context->audio_info.AudioCodecID = Multimedia::AUDIO_OTHERS; // 0x02;
			ROSE_PRINT(0x4,"<AUDIO CODEC> :  AUDIO_OTHERS");
		}
		break;

	/*	CODEC_ID_ADPCM_IMA_QT,
		CODEC_ID_ADPCM_IMA_WAV,
		CODEC_ID_ADPCM_IMA_DK3,
		CODEC_ID_ADPCM_IMA_DK4,
		CODEC_ID_ADPCM_IMA_WS,
		CODEC_ID_ADPCM_IMA_SMJPEG */
	case CODEC_ID_ADPCM_IMA_QT...CODEC_ID_ADPCM_IMA_SMJPEG:
	/* 	CODEC_ID_ADPCM_4XM,
		CODEC_ID_ADPCM_XA,
		CODEC_ID_ADPCM_ADX,
		CODEC_ID_ADPCM_EA,
		CODEC_ID_ADPCM_G726,
		CODEC_ID_ADPCM_CT,
		CODEC_ID_ADPCM_SWF,
		CODEC_ID_ADPCM_YAMAHA,
		CODEC_ID_ADPCM_SBPRO_4,
		CODEC_ID_ADPCM_SBPRO_3,
		CODEC_ID_ADPCM_SBPRO_2 */
	case CODEC_ID_ADPCM_MS...CODEC_ID_ADPCM_SBPRO_2:
		current_stream_info_context->audio_info.nSubId = current_avstream->codec->codec_id;
		current_stream_info_context->audio_info.AudioCodecID = Multimedia::AUDIO_ADPCM	; // 0x0B;
		ROSE_PRINT(0x4,"<AUDIO CODEC> :  AUDIO_ADPCM 1st");
		break;
	/*	CODEC_ID_ADPCM_THP,
		CODEC_ID_ADPCM_IMA_AMV,
		CODEC_ID_ADPCM_EA_R1,
		CODEC_ID_ADPCM_EA_R3,
		CODEC_ID_ADPCM_EA_R2,
		CODEC_ID_ADPCM_IMA_EA_SEAD,
		CODEC_ID_ADPCM_IMA_EA_EACS,
		CODEC_ID_ADPCM_EA_XAS,
		CODEC_ID_ADPCM_EA_MAXIS_XA */
	case CODEC_ID_ADPCM_THP...CODEC_ID_ADPCM_EA_MAXIS_XA:
		current_stream_info_context->audio_info.AudioCodecID = Multimedia::AUDIO_ADPCM	; // 0x0B;
		ROSE_PRINT(0x4,"<AUDIO CODEC> :  AUDIO_ADPCM 2nd");
		break;
	/* PCM codecs */
	case CODEC_ID_PCM_ALAW:
		current_stream_info_context->audio_info.nSubId = current_avstream->codec->codec_id;
		current_stream_info_context->audio_info.AudioCodecID = Multimedia::AUDIO_XLAW; // 0x09;
		ROSE_PRINT(0x4,"<AUDIO CODEC> :  AUDIO_PCM ALAW");
		break;
	case CODEC_ID_PCM_MULAW:
		current_stream_info_context->audio_info.nSubId = current_avstream->codec->codec_id;
		current_stream_info_context->audio_info.AudioCodecID = Multimedia::AUDIO_XLAW; // 0x09;
		ROSE_PRINT(0x4,"<AUDIO CODEC> :  AUDIO_PCM MULAW");
		break;
	case CODEC_ID_PCM_S16BE:
		current_stream_info_context->audio_info.nSubId = current_avstream->codec->codec_id; /* Samsung BUGFIX */
		current_stream_info_context->audio_info.AudioCodecID = Multimedia::AUDIO_PCM_S16; // 0x08;
		break;
	case CODEC_ID_PCM_S16LE:
	/*	CODEC_ID_PCM_U16LE,
		CODEC_ID_PCM_U16BE,
		CODEC_ID_PCM_S8,
		CODEC_ID_PCM_U8, */
	case CODEC_ID_PCM_U16LE...CODEC_ID_PCM_U8:
	/*
		CODEC_ID_PCM_S32LE,
		CODEC_ID_PCM_S32BE,
		CODEC_ID_PCM_U32LE,
		CODEC_ID_PCM_U32BE,
		CODEC_ID_PCM_S24LE,
		CODEC_ID_PCM_S24BE,
		CODEC_ID_PCM_U24LE,
		CODEC_ID_PCM_U24BE,
		CODEC_ID_PCM_S24DAUD, */
	case CODEC_ID_PCM_S32LE...CODEC_ID_PCM_S24DAUD:
		current_stream_info_context->audio_info.nSubId = current_avstream->codec->codec_id;
		current_stream_info_context->audio_info.AudioCodecID = Multimedia::AUDIO_XLAW; // 0x09;
		ROSE_PRINT(0x4,"<AUDIO CODEC> :  AUDIO_PCM");
		break;
	case CODEC_ID_MP2:
		current_stream_info_context->audio_info.AudioCodecID = Multimedia::AUDIO_MP3; // 0x03;
		ROSE_PRINT(0x4,"<AUDIO CODEC> :  AUDIO_MP2");
		break;
	case CODEC_ID_AAC:
		current_stream_info_context->audio_info.AudioCodecID = Multimedia::AUDIO_AAC; // 0x05;
		ROSE_PRINT(0x4,"<AUDIO CODEC> :  AUDIO_AAC");
		break;
	case CODEC_ID_AC3:
		current_stream_info_context->audio_info.AudioCodecID = Multimedia::AUDIO_AC3; // 0x06;
		ROSE_PRINT(0x4,"<AUDIO CODEC> :  AUDIO_AC3");
		break;
	case CODEC_ID_DTS:
		current_stream_info_context->audio_info.AudioCodecID = Multimedia::AUDIO_DTS; // 0x07;
		ROSE_PRINT(0x4,"<AUDIO CODEC> :  AUDIO_DTS");
		break;
	case CODEC_ID_MP3:
		current_stream_info_context->audio_info.AudioCodecID = Multimedia::AUDIO_MP3; // 0x03;
		ROSE_PRINT(0x4,"<AUDIO CODEC> :  AUDIO_MP3");
		break;
	case CODEC_ID_VORBIS:
		current_stream_info_context->audio_info.AudioCodecID = Multimedia::AUDIO_VORBIS; // 0x0E;
		ROSE_PRINT(0x4,"<AUDIO CODEC> :  AUDIO_VORBIS");
		break;
	case CODEC_ID_EAC3:
		current_stream_info_context->audio_info.AudioCodecID = Multimedia::AUDIO_EAC3; // 0x0C;
		ROSE_PRINT(0x4,"<AUDIO CODEC> :  AUDIO_EAC3");
		break;
	case CODEC_ID_WMAV1:
	case CODEC_ID_WMAV2:
	case CODEC_ID_WMAVOICE...CODEC_ID_WMALOSSLESS:
		current_stream_info_context->audio_info.AudioCodecID = Multimedia::AUDIO_WMA; // 0x0A;
		ROSE_PRINT(0x4,"<AUDIO CODEC> :  AUDIO_WMA %d", current_avstream->codec->codec_id );
		break;
	default:
		current_stream_info_context->audio_info.AudioCodecID = Multimedia::AUDIO_OTHERS; // 0x02;
		ROSE_PRINT(0x4,"<AUDIO CODEC> :  AUDIO_OTHERS");
		break;
	}

	ROSE_PRINT(0x4,"pMediaInfo->tStreams[index].tAudioInfo.nSubId = %d", current_stream_info_context->audio_info.nSubId );

	switch( current_avstream->codec->codec_id ){
	case CODEC_ID_PCM_S16LE:
	case CODEC_ID_PCM_S16BE:
	case CODEC_ID_PCM_U16LE:
	case CODEC_ID_PCM_U16BE:
		current_stream_info_context->audio_info.nBitRate = 
			(current_avstream->codec->sample_rate*current_avstream->codec->channels)*16;
		break;
	case CODEC_ID_PCM_S8:
	case CODEC_ID_PCM_U8:
	case CODEC_ID_PCM_MULAW:
	case CODEC_ID_PCM_ALAW:
		current_stream_info_context->audio_info.nBitRate = 
			(current_avstream->codec->sample_rate*current_avstream->codec->channels)*8;
		break;
	case CODEC_ID_PCM_S32LE:
	case CODEC_ID_PCM_S32BE:
	case CODEC_ID_PCM_U32LE:
	case CODEC_ID_PCM_U32BE:
		current_stream_info_context->audio_info.nBitRate = 
			(current_avstream->codec->sample_rate*current_avstream->codec->channels)*32;
		break;
	case CODEC_ID_PCM_S24LE:
	case CODEC_ID_PCM_S24BE:
	case CODEC_ID_PCM_U24LE:
	case CODEC_ID_PCM_U24BE:
	case CODEC_ID_PCM_S24DAUD:
		current_stream_info_context->audio_info.nBitRate = 
			(current_avstream->codec->sample_rate*current_avstream->codec->channels)*24;
		break;
	default:
		current_stream_info_context->audio_info.nBitRate = current_avstream->codec->bit_rate;
		break;
	}
	ROSE_PRINT(0x4,"pMediaInfo->tStreams[index].tAudioInfo.nBitRate = %d", current_stream_info_context->audio_info.nBitRate );
	
	current_stream_info_context->audio_info.nChannel = current_avstream->codec->channels;
	ROSE_PRINT(0x4,"pMediaInfo->tStreams[index].tAudioInfo.nChannel = %d", current_stream_info_context->audio_info.nChannel);

	// .text:00FBA544
	if( (media_info_context->container_info.nId == Multimedia::CONTAINER_MKV) &&
		(flag_mkv_audio & 1) ){
		for(int i = 0; i < mkv_demux_context->tracks.nb_elem; i++ ){
			if( ((unsigned)mkv_tracklist[i].stream->index == astream_index) &&
				(mkv_tracklist[i].type == MATROSKA_TRACK_TYPE_AUDIO) ){
				ROSE_PRINT(0x4,"*********************************************");
				ROSE_PRINT(0x4,"mkvaudiotrack->internal_samplerate = %d", (int)mkv_tracklist[i].audio.samplerate );
				ROSE_PRINT(0x4,"mkvaudiotrack->samplerate = %d", (int)mkv_tracklist[i].audio.out_samplerate );
				ROSE_PRINT(0x4,"*********************************************");
				current_stream_info_context->audio_info.nSampleRate = (int)mkv_tracklist[i].audio.samplerate;
				ROSE_PRINT(0x4,"pMediaInfo->tStreams[i].tAudioInfo.nSampleRate = %d", current_stream_info_context->audio_info.nSampleRate );
				break;
			}
		}
	} else {
		current_stream_info_context->audio_info.nSampleRate = current_avstream->codec->sample_rate;
		ROSE_PRINT(0x4,"pMediaInfo->tStreams[i].tAudioInfo.nSampleRate = %d", current_stream_info_context->audio_info.nSampleRate );
	}

	/* finally... */
	ROSE_PRINT(0x4,"[ffmpeg Audio Sample format = %d", current_avstream->codec->sample_fmt );
	current_stream_info_context->audio_info.nBlockAlign = current_avstream->codec->block_align;
	ROSE_PRINT(0x4,"pMediaInfo->tStreams[index].tAudioInfo.nBlockAlign = %d", current_stream_info_context->audio_info.nBlockAlign );
	
	current_stream_info_context->audio_info.nCodecTag = current_avstream->codec->codec_tag;
	ROSE_PRINT(0x4,"pMediaInfo->tStreams[index].tAudioInfo.nValidBitsPerSample =  %d", current_avstream->codec->bits_per_coded_sample );
	current_stream_info_context->audio_info.nValidBitsPerSample = current_avstream->codec->bits_per_coded_sample;
	current_stream_info_context->audio_info.nTotalPlayTime = media_info_context->container_info.nTotalPlayTime;
	ROSE_PRINT(0x4,"pMediaInfo->tStreams[index].tAudioInfo.nTotalPlayTime = %lld", current_stream_info_context->audio_info.nTotalPlayTime );
		
}

void CParsingMedia::SetSubtitleStreamInfo(AVFormatContext * ffMediaContext, unsigned int stream_index, Multimedia::_tMediaInfoContext * media_info_context)
{
	ROSE_PRINT(0x4,"[%d th] Subtitle Stream", stream_index );
	AVStream * current_avstream = ffMediaContext->streams[stream_index];
	ROSE_PRINT(0x4,"[Subtitle] Stream ID = %i.", current_avstream->id );
	ROSE_PRINT(0x4,"[Subtitle] Stream index = %i.", current_avstream->index );
	Multimedia::_tStreamInfoContext * current_stream_info_context = &media_info_context->stream_info[stream_index];
	current_stream_info_context->stream_type = Multimedia::STREAM_TYPE_SUBTITLE;				// offset 0x0238
	current_stream_info_context->stream_id = current_avstream->id;	// offset 0x023C
	ROSE_PRINT(0x4,"ffmpeg subtitle codec ID = %d", current_avstream->codec->codec_id);
	switch( current_avstream->codec->codec_id ){
	case CODEC_ID_DVD_SUBTITLE:
		current_stream_info_context->subtitle_info.nSubtitleFlag = 0;
		memcpy( current_stream_info_context->subtitle_info.cSubtitleLang, current_avstream->language, 4 );
		ROSE_PRINT(0x4,"DVD SUBTITLE : lang =[ %s ]", current_stream_info_context->subtitle_info.cSubtitleLang );
		break;
	case CODEC_ID_DVB_SUBTITLE:
		current_stream_info_context->subtitle_info.nSubtitleFlag = 1;
		memcpy( current_stream_info_context->subtitle_info.cSubtitleLang, current_avstream->language, 4 );
		ROSE_PRINT(0x4,"DVD SUBTITLE : lang =[ %s ]", current_stream_info_context->subtitle_info.cSubtitleLang );
		break;
	case CODEC_ID_TEXT:
		current_stream_info_context->subtitle_info.nSubtitleFlag = 2;
		memcpy( current_stream_info_context->subtitle_info.cSubtitleLang, current_avstream->language, 4 );
		ROSE_PRINT(0x4,"TEXT SUBTITLE : lang =[ %s ]", current_stream_info_context->subtitle_info.cSubtitleLang );
		break;
	case CODEC_ID_XSUB:
		current_stream_info_context->subtitle_info.nSubtitleFlag = 3;
		memcpy( current_stream_info_context->subtitle_info.cSubtitleLang, current_avstream->language, 4 );
		ROSE_PRINT(0x4,"XSUB SUBTITLE : lang =[ %s ]", current_stream_info_context->subtitle_info.cSubtitleLang );
		break;
    /* CODEC_ID_SSA,
		CODEC_ID_MOV_TEXT */
	default:
		break;
	}
}

void CParsingMedia::SetVideoStreamInfo(AVFormatContext * ffMediaContext, unsigned int stream_index, Multimedia::_tMediaInfoContext * media_info_context)
{

	AVStream * current_avstream = ffMediaContext->streams[stream_index];
	Multimedia::_tStreamInfoContext * current_stream_info_context = &media_info_context->stream_info[stream_index];
	current_stream_info_context->stream_id = current_avstream->id;	// offset 0x023C
	current_stream_info_context->stream_type = Multimedia::STREAM_TYPE_VIDEO;				// offset 0x0238

	ROSE_PRINT(0x4,"[%d th] Video Stream", stream_index);
	if( media_info_context->container_info.nId == Multimedia::CONTAINER_MKV ){
		ROSE_PRINT(0x4,"MKV DEMUX Context");
		if( ffMediaContext->priv_data ){
			ROSE_PRINT(0x4,"MKV has priv_data.");
			MatroskaDemuxContextOld * mkv_demux_context = (MatroskaDemuxContextOld *)ffMediaContext->priv_data;
			S_MatroskaTrack * mkv_tracklist = (S_MatroskaTrack *)mkv_demux_context->tracks.elem; 
			for(int i = 0; i < mkv_demux_context->tracks.nb_elem; i++ ){
				if( mkv_tracklist[i].type == MATROSKA_TRACK_TYPE_VIDEO ){
					current_stream_info_context->video_info.nPid = (int)mkv_tracklist[i].num;
					break;
				}
			}
		}
	} else {
		current_stream_info_context->video_info.nPid = current_avstream->id;
	}
	
	ROSE_PRINT(0x4,"pMediaInfo->tStreams[index].tVideoInfo.nPid = %d", current_stream_info_context->video_info.nPid );
	ROSE_PRINT(0x4,"ffmpeg video codec ID = %d", current_avstream->codec->codec_id);
	switch( current_avstream->codec->codec_id ){
	case CODEC_ID_NONE:
		current_stream_info_context->video_info.VideoCodecID = Multimedia::VIDEO_NONE; // 0x00;
		ROSE_PRINT(0x4,"<VIDEO CODEC> :  VIDEO_NONE");
		break;
	case CODEC_ID_MPEG1VIDEO:
		current_stream_info_context->video_info.VideoCodecID = Multimedia::VIDEO_MPEG1; // 0x04;
		ROSE_PRINT(0x4,"<VIDEO CODEC> :  VIDEO_MPEG1");
		break;
	case CODEC_ID_MPEG2VIDEO:
		current_stream_info_context->video_info.VideoCodecID = Multimedia::VIDEO_MPEG2; //0x05;
		ROSE_PRINT(0x4,"<VIDEO CODEC> :  VIDEO_MPEG2");
		break;
	case CODEC_ID_MJPEG:
		current_stream_info_context->video_info.VideoCodecID = Multimedia::VIDEO_MJPEG; //0x0F;
		ROSE_PRINT(0x4,"<VIDEO CODEC> :  VIDEO_MJPEG");
		break;
	case CODEC_ID_MJPEGB:
		current_stream_info_context->video_info.VideoCodecID = Multimedia::VIDEO_MJPEGB; //0x10;
		ROSE_PRINT(0x4,"<VIDEO CODEC> :  VIDEO_MJPEGB");
		break;
	case CODEC_ID_MPEG4:
		switch( current_avstream->codec->codec_tag ){
		case 0x58564944:		/* DIVX */
			current_stream_info_context->video_info.VideoCodecID = Multimedia::VIDEO_DIVX; //0x0B;
			ROSE_PRINT(0x4,"<VIDEO CODEC> :  VIDEO_DIVX");
			break;
		case 0x44495658:	/* XVID */
		case 0x64697678:	/* xvid */
			current_stream_info_context->video_info.VideoCodecID = Multimedia::VIDEO_XVID; //0x0D;
			ROSE_PRINT(0x4,"<VIDEO CODEC> :  VIDEO_XVID");
			break;
		case 0x30355844:	/* DX50 */
			current_stream_info_context->video_info.VideoCodecID = Multimedia::VIDEO_DX50; //0x0C;
			ROSE_PRINT(0x4,"<VIDEO CODEC> :  VIDEO_DX50");
			break;
		default:
			current_stream_info_context->video_info.VideoCodecID = Multimedia::VIDEO_MPEG4; //0x06;
			ROSE_PRINT(0x4,"<VIDEO CODEC> :  VIDEO_MPEG4");
			break;
		}
		break;
	case CODEC_ID_MSMPEG4V1:
	case CODEC_ID_MSMPEG4V2:
		current_stream_info_context->video_info.VideoCodecID = Multimedia::VIDEO_MSMPEGV; //0x02;
		ROSE_PRINT(0x4,"<VIDEO CODEC> :  VIDEO_MSMPEGV");
		break;
	case CODEC_ID_MSMPEG4V3:
		if( current_avstream->codec->codec_tag == 0x33564944 ){ /* DIV3 */
			current_stream_info_context->video_info.VideoCodecID = Multimedia::VIDEO_DIVX; //0x0B;
			ROSE_PRINT(0x4,"<VIDEO CODEC> :  VIDEO_DIVX");
		} else {
			current_stream_info_context->video_info.VideoCodecID = Multimedia::VIDEO_MSMPEGV3; //0x03;
			ROSE_PRINT(0x4,"<VIDEO CODEC> :  VIDEO_MSMPEGV3");
		}
		break;
	case CODEC_ID_WMV1:
		current_stream_info_context->video_info.VideoCodecID = Multimedia::VIDEO_WMV; //0x09;
		ROSE_PRINT(0x4,"<VIDEO CODEC> :  VIDEO_WMV");
		break;
	case CODEC_ID_H264:
		current_stream_info_context->video_info.VideoCodecID = Multimedia::VIDEO_H264; //0x07;
		ROSE_PRINT(0x4,"<VIDEO CODEC> :  VIDEO_H264");
		break;
	case CODEC_ID_XVID:
		current_stream_info_context->video_info.VideoCodecID = Multimedia::VIDEO_XVID; //0x0D;
		ROSE_PRINT(0x4,"<VIDEO CODEC> :  VIDEO_XVID");
		break;
	case CODEC_ID_VC1:
		current_stream_info_context->video_info.VideoCodecID = Multimedia::VIDEO_VC1; //0x08;
		ROSE_PRINT(0x4,"<VIDEO CODEC> :  VIDEO_VC1");
		break;
	case CODEC_ID_WMV3:
		current_stream_info_context->video_info.VideoCodecID = Multimedia::VIDEO_WMV9; // 0x0A;
		ROSE_PRINT(0x4,"<VIDEO CODEC> :  VIDEO_WMV9");
		break;
	case CODEC_ID_INDEO5:
		if( current_avstream->codec->codec_tag == 0x30355649 ){ /* IV50 */
			current_stream_info_context->video_info.VideoCodecID = Multimedia::VIDEO_DIVX; // 0x0B;
			ROSE_PRINT(0x4,"<VIDEO CODEC> :  <VIDEO CODEC> :  VIDEO_DIVX (PRO 5.0.2)");
		}
		break;
	default:
		current_stream_info_context->video_info.VideoCodecID = Multimedia::VIDEO_UNSUPPORTED; // 0x01;
		ROSE_PRINT(0x4,"<VIDEO CODEC> :  VIDEO_NOT_SUPPORTED");
		break;
	}
	
	current_stream_info_context->video_info.nSubId = current_avstream->codec->sub_id;
	ROSE_PRINT(0x4,"pMediaInfo->tStreams[index].tVideoInfo.nSubId = %d", current_stream_info_context->video_info.nSubId);
	current_stream_info_context->video_info.nFrameCount = current_avstream->nb_frames;
	ROSE_PRINT(0x4,"pMediaInfo->tStreams[index].tVideoInfo.nFrameCount = %lld", current_stream_info_context->video_info.nFrameCount);
	current_stream_info_context->video_info.nFrameHeight = current_avstream->codec->height;
	ROSE_PRINT(0x4,"pMediaInfo->tStreams[index].tVideoInfo.nFrameHeight = %d", current_stream_info_context->video_info.nFrameHeight);
	current_stream_info_context->video_info.nFrameWidth = current_avstream->codec->width;
	ROSE_PRINT(0x4,"pMediaInfo->tStreams[index].tVideoInfo.nFrameWidth = %d", current_stream_info_context->video_info.nFrameWidth);

	if( (current_avstream->r_frame_rate.den) &&
		(current_avstream->r_frame_rate.num) ){
		current_stream_info_context->video_info.nFrameRate = (float)( (double)current_avstream->r_frame_rate.num / (double)current_avstream->r_frame_rate.den );
	} else {
		current_stream_info_context->video_info.nFrameRate = (float) ( (double (1)) / ((double)current_avstream->codec->time_base.num / (double)current_avstream->codec->time_base.den) );
	}
	ROSE_PRINT(0x4,"pMediaInfo->tStreams[index].tVideoInfo.nFrameRate = %f", current_stream_info_context->video_info.nFrameRate);

	current_stream_info_context->video_info.nTotalPlayTime = media_info_context->container_info.nTotalPlayTime;
	ROSE_PRINT(0x4,"pMediaInfo->tStreams[index].tVideoInfo.nTotalPlayTime = %lld", current_stream_info_context->video_info.nTotalPlayTime );
}

void CParsingMedia::SetContainerInfo(AVFormatContext * ffMediaContext, char const * stream_name, Multimedia::_tMediaInfoContext * media_info_context)
{
	ROSE_PRINT(0x4,"[CParsingMedia] %s In", __FUNCTION__ );
	ROSE_PRINT(0x4,"ffMediaContext->iformat->name = %s", ffMediaContext->iformat->name );

	for( ; ; ){
		if( strcmp(ffMediaContext->iformat->name, "avi") == 0 ){
			ROSE_PRINT(0x4,"<AVI> format");
			media_info_context->container_info.nId = Multimedia::CONTAINER_AVI; // 0x09;
			break;
		}
		if( strcmp(ffMediaContext->iformat->name, "flv") == 0 ){
			ROSE_PRINT(0x4,"<flv> format");
			break;
		}
		if( strcmp(ffMediaContext->iformat->name, "mpegts") == 0 ){
			ROSE_PRINT(0x4,"<mpegts> format");
			// .text:00FBB0F0
			ROSE_PRINT(0x4,"TODO: CParsingMedia::AnalyseTSstream(char  const*)");
			if( AnalyseTSstream( stream_name ) == 1 ){
				ROSE_PRINT(0x4,"Container : TTS");
				media_info_context->container_info.nId = Multimedia::CONTAINER_MPEGTTS; // = 0x0D;
			} else {
				ROSE_PRINT(0x4,"Container : TS");
				media_info_context->container_info.nId = Multimedia::CONTAINER_MPEGTS; // = 0x05;
			}
			break;
		}

		if( strncmp(ffMediaContext->iformat->name, "matroska", 8) == 0 ){
			ROSE_PRINT(0x4,"<matroska(mkv)> format");
			media_info_context->container_info.nId = Multimedia::CONTAINER_MKV; // = 0x0E;
			break;
		}
		if( strcmp(ffMediaContext->iformat->name, "asf") == 0 ){
			ROSE_PRINT(0x4,"<asf> format");
			media_info_context->container_info.nId = Multimedia::CONTAINER_ASF; // = 0x06;
			break;
		}
		if( strcmp(ffMediaContext->iformat->name, "mp3") == 0 ){
			ROSE_PRINT(0x4,"<mp3> format");
			media_info_context->container_info.nId = Multimedia::CONTAINER_MP3; // = 0x0B;
			break;
		}
		if( strcmp(ffMediaContext->iformat->name, "mov,mp4,m4a,3gp,3g2,mj2") == 0 ){
			ROSE_PRINT(0x4,"<mov,mp4,m4a,3gp,3g2,mj2> format");
			media_info_context->container_info.nId = Multimedia::CONTAINER_MP4; // = 0x0C;
			break;
		}
		if( strcmp(ffMediaContext->iformat->name, "mpeg") == 0 ){
			ROSE_PRINT(0x4,"<mpeg> format");
			media_info_context->container_info.nId = Multimedia::CONTAINER_MPEG; // = 0x04;
			break;
		}
		media_info_context->container_info.nId = Multimedia::CONTAINER_NONE; // = 0x00;
		break;
	}
	ROSE_PRINT(0x4,"pMediaInfo->tContainerInfo.nId = %d", media_info_context->container_info.nId);
	media_info_context->nBitRate = ffMediaContext->bit_rate;
	ROSE_PRINT(0x4,"pMediaInfo->nBitRate = %d", media_info_context->nBitRate );
	if( media_info_context->container_info.nId == Multimedia::CONTAINER_MPEGTS ){
		for( unsigned int i = 0; i < ffMediaContext->nb_streams; i++ ){
			if( ffMediaContext->streams[i]->codec->codec_type == AVMEDIA_TYPE_VIDEO ){
				media_info_context->nPcrPid = ((PESContext *)(ffMediaContext->streams[i]->priv_data))->pcr_pid;
				break;
			}
		}
	}
	ROSE_PRINT(0x4,"pMediaInfo->nPcrPid = %d", media_info_context->nPcrPid );
	media_info_context->nStreams = ffMediaContext->nb_streams;
	ROSE_PRINT(0x4,"pMediaInfo->nStreams=pFormatContext->nb_streams =%u", media_info_context->nStreams );
	strcpy( media_info_context->file_info.filename, ffMediaContext->filename );
	ROSE_PRINT(0x4,"pMediaInfo->tFileInfo.fileName = %s", media_info_context->file_info.filename);
	media_info_context->file_info.filesize = ffMediaContext->file_size;
	ROSE_PRINT(0x4,"pMediaInfo->tFileInfo.fileSize = %lld", media_info_context->file_info.filesize );

	ROSE_PRINT(0x4,"[CParsingMedia] %s Out", __FUNCTION__ );

}

/* use external FFMPEG libraries */
void CParsingMedia::PrepareParsingExtFfmpeg(void)
{
	char library_path[256];

	ROSE_PRINT(0x4,"[CParsingMedia] %s In", __FUNCTION__ );

	PRINTF("External library path = '%s'", path_ext_ffmpeg );

 	snprintf(library_path, sizeof(library_path), "%s%s", path_ext_ffmpeg, "libavutil.so.50"); 
	hLib_avutil		= dlopen( library_path, RTLD_LAZY );
	dlError			= dlerror();
	if( hLib_avutil == NULL ){
		ROSE_PRINT(0x4000,"[%s dlopen error]  %s", library_path, dlError );
		goto exit_close_avutil;
	}
 	snprintf(library_path, sizeof(library_path), "%s%s", path_ext_ffmpeg, "libavcore.so.0"); 
	hLib_avcore		= dlopen( library_path, RTLD_LAZY );
	dlError			= dlerror();
	if( hLib_avcore == NULL ){
		ROSE_PRINT(0x4000,"[%s dlopen error]  %s", library_path, dlError );
		// goto exit_close_avutil;
	}
 	snprintf(library_path, sizeof(library_path), "%s%s", path_ext_ffmpeg, "libavcodec.so.52"); 
	hLib_avcodec		= dlopen( library_path, RTLD_LAZY );
	dlError			= dlerror();
	if( hLib_avcodec == NULL ){
		ROSE_PRINT(0x4000,"[%s dlopen error]  %s", library_path, dlError );
		goto exit_close_avcodec;
	}
 	snprintf(library_path, sizeof(library_path), "%s%s", path_ext_ffmpeg, "libavformat.so.52"); 
	hLib_avformat		= dlopen( library_path, RTLD_LAZY );
	dlError			= dlerror();
	if( hLib_avformat == NULL ){
		ROSE_PRINT(0x4000,"[%s dlopen error]  %s", library_path, dlError );
		goto exit_close_avformat;
	}

	av_register_all222 = (av_register_all_t)dlsym(hLib_avformat,"av_register_all");
	if( av_register_all222 == NULL ){
		ROSE_PRINT(0x4000,"[av_register_all222 dlsym error]");
		goto exit_close_avformat;
	}
	av_open_input_file222 = (av_open_input_file_t)dlsym(hLib_avformat,"av_open_input_file");
	if( av_open_input_file222 == NULL ){
		ROSE_PRINT(0x4000,"[av_open_input_file222 dlsym error]");
		goto exit_close_avformat;
	}
	av_close_input_file222 = (av_close_input_file_t)dlsym(hLib_avformat,"av_close_input_file");
	if( av_close_input_file222 == NULL ){
		ROSE_PRINT(0x4000,"[av_close_input_file222 dlsym error]");
		goto exit_close_avformat;
	}
	av_find_stream_info222 = (av_find_stream_info_t)dlsym(hLib_avformat,"av_find_stream_info");
	if( av_find_stream_info222 == NULL ){
		ROSE_PRINT(0x4000,"[av_find_stream_info222 dlsym error]");
		goto exit_close_avformat;
	}
	dump_format222 = (dump_format_t)dlsym(hLib_avformat,"dump_format");
	if( dump_format222 == NULL ){
		ROSE_PRINT(0x4000,"[dump_format222 dlsym error]");
		goto exit_close_avformat;
	}
	getMpeg4AACObjetType222 = (getMpeg4AACObjetType_t)dlsym(hLib_avformat,"getMpeg4AACObjetType");
	if( getMpeg4AACObjetType222 == NULL ){
		ROSE_PRINT(0x4000,"[getMpeg4AACObjetType222 dlsym error]");
		goto exit_close_avformat;
	}
	getMpeg4AACSBR222 = (getMpeg4AACSBR_t)dlsym(hLib_avformat,"getMpeg4AACSBR");
	if( getMpeg4AACSBR222 == NULL ){
		ROSE_PRINT(0x4000,"[getMpeg4AACSBR222 dlsym error]");
		goto exit_close_avformat;
	}

	/* all OK */
	ROSE_PRINT(0x4,"[CParsingMedia] %s Out", __FUNCTION__ );
	return;

exit_close_avformat:
	dlclose( hLib_avformat );
exit_close_avcodec:
	dlclose( hLib_avcodec );
exit_close_avutil:
	dlclose( hLib_avutil );
	return;
}

void CParsingMedia::PrepareParsing(void)
{	
	ROSE_PRINT(0x4,"[CParsingMedia] %s In", "PrepareParsing" );

	hLib_avutil		= dlopen( "libavutil.so.49", RTLD_LAZY );
	dlError			= dlerror();
	if( hLib_avutil == NULL ){
		ROSE_PRINT(0x4000,"[libavutil.so dlopen error]  %s", dlError );
		goto exit_close_avutil;
	}
	hLib_avcodec	= dlopen( "libavcodec.so.51", RTLD_LAZY );
	dlError			= dlerror();
	if( hLib_avcodec == NULL ){
		ROSE_PRINT(0x4000,"[libavcodec.so dlopen error]  %s", dlError );
		goto exit_close_avcodec;
	}
	hLib_avformat	= dlopen( "libavformat.so.52", RTLD_LAZY );
	dlError			= dlerror();
	if( hLib_avformat == NULL ){
		ROSE_PRINT(0x4000,"[libavformat.so dlopen error]  %s", dlError );
		goto exit_close_avformat;
	}

	av_register_all222 = (av_register_all_t)dlsym(hLib_avformat,"av_register_all");
	if( av_register_all222 == NULL ){
		ROSE_PRINT(0x4000,"[av_register_all222 dlsym error]");
		goto exit_close_avformat;
	}
	av_open_input_file222 = (av_open_input_file_t)dlsym(hLib_avformat,"av_open_input_file");
	if( av_open_input_file222 == NULL ){
		ROSE_PRINT(0x4000,"[av_open_input_file222 dlsym error]");
		goto exit_close_avformat;
	}
	av_close_input_file222 = (av_close_input_file_t)dlsym(hLib_avformat,"av_close_input_file");
	if( av_close_input_file222 == NULL ){
		ROSE_PRINT(0x4000,"[av_close_input_file222 dlsym error]");
		goto exit_close_avformat;
	}
	av_find_stream_info222 = (av_find_stream_info_t)dlsym(hLib_avformat,"av_find_stream_info");
	if( av_find_stream_info222 == NULL ){
		ROSE_PRINT(0x4000,"[av_find_stream_info222 dlsym error]");
		goto exit_close_avformat;
	}
	dump_format222 = (dump_format_t)dlsym(hLib_avformat,"dump_format");
	if( dump_format222 == NULL ){
		ROSE_PRINT(0x4000,"[dump_format222 dlsym error]");
		goto exit_close_avformat;
	}
	getMpeg4AACObjetType222 = (getMpeg4AACObjetType_t)dlsym(hLib_avformat,"getMpeg4AACObjetType");
	if( getMpeg4AACObjetType222 == NULL ){
		ROSE_PRINT(0x4000,"[getMpeg4AACObjetType222 dlsym error]");
		goto exit_close_avformat;
	}
	getMpeg4AACSBR222 = (getMpeg4AACSBR_t)dlsym(hLib_avformat,"getMpeg4AACSBR");
	if( getMpeg4AACSBR222 == NULL ){
		ROSE_PRINT(0x4000,"[getMpeg4AACSBR222 dlsym error]");
		goto exit_close_avformat;
	}

	/* all OK */
	ROSE_PRINT(0x4,"[CParsingMedia] %s Out", "PrepareParsing" );
	return;

exit_close_avformat:
	dlclose( hLib_avformat );
exit_close_avcodec:
	dlclose( hLib_avcodec );
exit_close_avutil:
	dlclose( hLib_avutil );
	return;
}

//struct ts_analyse_t {
//	int mTmp[0x200];
//};

void CParsingMedia::LoadImportsFfmpeg( struct ffmpeg_interface_t  * ff )
{

	ROSE_PRINT(0x4,"[CParsingMedia] %s In", __FUNCTION__ );

	/* populate ffmpeg_interface_t structure */
	if( ff->initialised ){
		ROSE_PRINT(0x4004,"[CParsingMedia] %s already initialised.", __FUNCTION__ );
		goto LoadImportsFfmpeg_Out;
	}
	ff->av_register_all = (int (*)(void))dlsym( hLib_avformat, "av_register_all" );
	if( ff->av_register_all == NULL ){
		ROSE_PRINT(0x4000,"[ff->av_register_all dlsym error]");
	}
	ff->av_init_packet = (void (*)(AVPacket*))dlsym( hLib_avformat, "av_init_packet" );
	if( ff->av_init_packet == NULL ){
		ROSE_PRINT(0x4000,"[ff->av_init_packet dlsym error]");
	}
	ff->avcodec_register_all = (void (*)(void))dlsym( hLib_avcodec, "avcodec_register_all" );
	if( ff->avcodec_register_all == NULL ){
		ROSE_PRINT(0x4000,"[ff->avcodec_register_all dlsym error]");
	}
	ff->avcodec_find_decoder = (AVCodec* (*)(enum CodecID))dlsym( hLib_avcodec, "avcodec_find_decoder" );
	if( ff->avcodec_find_decoder == NULL ){
		ROSE_PRINT(0x4000,"[ff->avcodec_find_decoder dlsym error]");
	}
	ff->avcodec_alloc_context = (AVCodecContext* (*)())dlsym( hLib_avcodec, "avcodec_alloc_context" );
	if( ff->avcodec_alloc_context == NULL ){
		ROSE_PRINT(0x4000,"[ff->avcodec_alloc_context dlsym error]");
	}
	ff->avcodec_open = (int (*)(AVCodecContext*, AVCodec*))dlsym( hLib_avcodec, "avcodec_open" );
	if( ff->avcodec_open == NULL ){
		ROSE_PRINT(0x4000,"[ff->avcodec_open dlsym error]");
	}
	ff->avcodec_close = (int (*)(AVCodecContext*))dlsym( hLib_avcodec, "avcodec_close" );
	if( ff->avcodec_close == NULL ){
		ROSE_PRINT(0x4000,"[ff->avcodec_close dlsym error]");
	}
	ff->avcodec_decode_audio3 = (int (*)(AVCodecContext*, int16_t*, int*, AVPacket*))dlsym( hLib_avcodec, "avcodec_decode_audio3" );
	if( ff->avcodec_decode_audio3 == NULL ){
		ROSE_PRINT(0x4000,"[ff->avcodec_decode_audio3 dlsym error]");
	}

	ff->av_audio_resample_init = (ReSampleContext * (*)(int, int, int, int, AVSampleFormat, AVSampleFormat, int, int, int, double))dlsym( hLib_avcodec, "av_audio_resample_init" );
	if( ff->av_audio_resample_init == NULL ){
		ROSE_PRINT(0x4000,"[ff->av_audio_resample_init dlsym error]");
	}
	ff->audio_resample = (int (*)(ReSampleContext*, short int*, short int*, int))dlsym( hLib_avcodec, "audio_resample" );
	if( ff->audio_resample == NULL ){
		ROSE_PRINT(0x4000,"[ff->audio_resample dlsym error]");
	}
	ff->audio_resample_close = (int (*)(ReSampleContext*))dlsym( hLib_avcodec, "audio_resample_close" );
	if( ff->audio_resample_close == NULL ){
		ROSE_PRINT(0x4000,"[ff->audio_resample_close dlsym error]");
	}
	ff->avcodec_encode_audio = (int (*)(AVCodecContext *, uint8_t *, int , const short *))dlsym( hLib_avcodec, "avcodec_encode_audio" );
	if( ff->avcodec_encode_audio == NULL ){
		ROSE_PRINT(0x4000,"[ff->avcodec_encode_audio dlsym error]");
	}
	ff->avcodec_find_encoder = (AVCodec* (*)(enum CodecID))dlsym( hLib_avcodec, "avcodec_find_encoder" );
	if( ff->avcodec_find_encoder == NULL ){
		ROSE_PRINT(0x4000,"[ff->avcodec_find_encoder dlsym error]");
	}

	ff->initialised = 1;

LoadImportsFfmpeg_Out:
	ROSE_PRINT(0x4,"[CParsingMedia] %s Out", __FUNCTION__ );

}

/*
int CParsingMedia::AnalyseTSstream(char  const* filename )
{
	struct ts_analyse_t * buffer1;
	struct ts_analyse_t * buffer2;
	int retval = -1;
	int status, res1, res2, res3, tmp;

	FILE * fd_file = fopen64( filename, "rb" );
	if( fd_file == NULL )
		return retval;

	buffer1 = new(ts_analyse_t);
	if( buffer1 == NULL )
		goto analyze_close_exit;
	if( fread((void *)buffer1, 1, sizeof(ts_analyse_t), fd_file) != sizeof(ts_analyse_t) )
		goto analyze_delete_buffer1;
	buffer2 = new(ts_analyse_t);
	if( buffer2 == NULL )
		goto analyze_delete_buffer1;
	memcpy( buffer2, buffer1, sizeof(ts_analyse_t) );
	res1 = TSAnalyze(buffer2, 0x758, 0xBC, &status );
	res2 = TSAnalyze(buffer2, 0x780, 0xC0, &status );
	res3 = TSAnalyze(buffer2, 0x7F8, 0xCC, &status );
	
	if( (res1 > res3) && (res1 > res2) ){
	} else {
		if( res1 >= res2 )
			tmp = 0;
		else
			tmp = 1;
		if( res2 <= res3 )
			tmp = 0;
	}
...
...
	free( buffer2 );
analyze_delete_buffer1:
	free( buffer1 );
analyze_close_exit:
	fclose( fd_file );
	return retval;
}
*/
int CParsingMedia::GetMediaTypeInfo(char  const* file_name, Multimedia::_tMediaInfoContext * media_info_context )
{
    AVFormatContext *av_format_context;
	int retval;

	ROSE_PRINT(0x4,"[CParsingMedia] %s In", "GetMediaTypeInfo" );

	av_register_all222();
	retval = av_open_input_file222(&av_format_context, file_name, NULL, 0, NULL);
	if( retval != 0 ){
		ROSE_PRINT(0x4000,"[GetMediaTypeInfoUsingFFMpeg] OPENING  IS FAILED [%d]", retval );
		return 0;
	}
	ROSE_PRINT(0x4, "[GetMediaTypeInfoUsingFFMpeg] OPEN SUCCESS");
	if( av_find_stream_info222(av_format_context) < 0 ){
		if( (av_format_context->iformat == NULL) ||
			(av_format_context->iformat->name == NULL ) ||
			(strcmp(av_format_context->iformat->name, "mpegts")) ){
			ROSE_PRINT(0x4000, "[GetMediaTypeInfoUsingFFMpeg] STREAM INFO ERROR" );
			av_close_input_file222( av_format_context );
			return 0;
		} else {
			for( unsigned int i = 0; i < av_format_context->nb_streams; i++ ){
				if( av_format_context->streams[i]->codec == NULL ){
					ROSE_PRINT(0x4000, "[GetMediaTypeInfoUsingFFMpeg] STREAM INFO ERROR(TS CASE)" );
					av_close_input_file222( av_format_context );
					return 0;
				}
				ROSE_PRINT(0x4, "PARSING PASSED(TS CASE)");
			}
		}
	} else {
		ROSE_PRINT(0x4, "===============[DUMP FORMAT]==============");
		dump_format222( av_format_context, 0, file_name, 0 );
		ROSE_PRINT(0x4, "==========================================");
	}
	memset( media_info_context->file_info.filename, 0, 4 );
	ROSE_PRINT(0x4, "inputformat name = %s", av_format_context->iformat->name );
	ROSE_PRINT(0x4, "inputformat extension = %s", av_format_context->iformat->extensions );
	SetContainerInfo( av_format_context, file_name, media_info_context );
	media_info_context->nVideoStream = 0;
	media_info_context->nAudioStream = 0;
	media_info_context->nSubtitleStream = 0;
	for(unsigned int i = 0; i < av_format_context->nb_streams; i++ ){
		switch( av_format_context->streams[i]->codec->codec_type ){
		case AVMEDIA_TYPE_VIDEO:	// 0
			/* media_info_context->nVideoStream = (i+1); */
			media_info_context->nVideoStream++;
			break;
		case AVMEDIA_TYPE_AUDIO:	// 1
			media_info_context->nAudioStream++;
			break;
		case AVMEDIA_TYPE_SUBTITLE:	// 3
			media_info_context->nSubtitleStream++;
			break;
		default:
			break;
		}
	}
	ROSE_PRINT(0x4, "pMediaInfo->nVideoStream = %d", media_info_context->nVideoStream );
	ROSE_PRINT(0x4, "pMediaInfo->nAudioStream = %d", media_info_context->nAudioStream );

#if 0
	/* I don't get the point of this... */
	/* 00FBB490...00FBB4F8 */
	/* BUG: Samsung code badly fails on DVB stream handling */
	int video_stream_id = -1;
	int audio_stream_id = -1;
	int video_stream_index = 10000;
	int audio_stream_index = 10000;
	for( unsigned int i = 0; i < av_format_context->nb_streams; i++ ){
		switch( av_format_context->streams[i]->codec->codec_type ){
		case AVMEDIA_TYPE_VIDEO:
			if( av_format_context->streams[i]->id < video_stream_index ){
				video_stream_index = av_format_context->streams[i]->id;
				video_stream_id = i;
			}
			break;
		case AVMEDIA_TYPE_AUDIO:
			if( av_format_context->streams[i]->id < audio_stream_index ){
				audio_stream_index = av_format_context->streams[i]->id;
				audio_stream_id = i;
			}
			break;
		default:
			break;
		}
	}
#else
	int video_stream_id = -1;
	int audio_stream_id = -1;
	int video_stream_index = 10000;
	int audio_stream_index = 10000;
	if( av_format_context->nb_programs ){
		for(unsigned int j = 0; j < av_format_context->nb_programs; j++ ){
			for(unsigned int k = 0; k < av_format_context->programs[j]->nb_stream_indexes; k++ ){
				int i = av_format_context->programs[j]->stream_index[k];
				switch( av_format_context->streams[i]->codec->codec_type ){
				case AVMEDIA_TYPE_VIDEO:
					if( av_format_context->streams[i]->id < video_stream_index ){
						video_stream_index = av_format_context->streams[i]->id;
						video_stream_id = i;
					}
					break;
				case AVMEDIA_TYPE_AUDIO:
					if( av_format_context->streams[i]->id < audio_stream_index ){
						audio_stream_index = av_format_context->streams[i]->id;
						audio_stream_id = i;
					}
					break;
				default:
					break;
				}
			}
		}
	} else {
		for( unsigned int i = 0; i < av_format_context->nb_streams; i++ ){
			switch( av_format_context->streams[i]->codec->codec_type ){
			case AVMEDIA_TYPE_VIDEO:
				if( av_format_context->streams[i]->id < video_stream_index ){
					video_stream_index = av_format_context->streams[i]->id;
					video_stream_id = i;
				}
				break;
			case AVMEDIA_TYPE_AUDIO:
				if( av_format_context->streams[i]->id < audio_stream_index ){
					audio_stream_index = av_format_context->streams[i]->id;
					audio_stream_id = i;
				}
				break;
			default:
				break;
			}
		}
	}
#endif

	/* some debug output */
	PRINTF("Calculating nTotalPlayTime..." );
	PRINTF("video_stream_id = %d", video_stream_id );
	PRINTF("audio_stream_id = %d", audio_stream_id );

	if( (int)(av_format_context->duration >> 32) < 0 ){
		/* loc_FBB834 */
		// if( ((audio_stream_id != -1) && (video_stream_id != -1))  ||
		if( ((audio_stream_id > -1) && (video_stream_id > -1)) &&
			((av_format_context->streams[video_stream_id]->codec->bit_rate > 0) && 
			(av_format_context->streams[audio_stream_id]->codec->bit_rate > 0)) ){

			PRINTF("(1) video bit_rate = %d", av_format_context->streams[video_stream_id]->codec->bit_rate );
			PRINTF("(1) audio bit_rate = %d", av_format_context->streams[audio_stream_id]->codec->bit_rate );

			media_info_context->container_info.nTotalPlayTime = av_format_context->file_size * 8 / 
				(((av_format_context->streams[video_stream_id]->codec->bit_rate + av_format_context->streams[audio_stream_id]->codec->bit_rate) / 1000) << 10);
		} else {
			if( (video_stream_id >= 0) &&
				(av_format_context->streams[video_stream_id]->codec->bit_rate > 0) ){
				ROSE_PRINT( 0x4004, "Audio  Value is empty.." );
				/* overall average bit rate = file size in bytes * bits per byte / duration in seconds */
				PRINTF("(2) video bit_rate = %d", av_format_context->streams[video_stream_id]->codec->bit_rate );
				media_info_context->container_info.nTotalPlayTime = av_format_context->file_size * 8 / ((av_format_context->streams[video_stream_id]->codec->bit_rate / 1000) << 10);
			} else {
				ROSE_PRINT( 0x4004, "Audio and Video Codec Bitrate Value is empty.." );
			}
		}
	} else {
		media_info_context->container_info.nTotalPlayTime = av_format_context->duration / (int64_t)1000000;
	}
	ROSE_PRINT(0x4, "pMediaInfo->tContainerInfo.nTotalPlayTime =%lld", media_info_context->container_info.nTotalPlayTime );

	if( av_format_context->nb_programs > 0 ){
		ROSE_PRINT(0x4, "Processing DVB streams...");
#if 0
		/* BUG: original Samsung code (read and enjoy)... */
		/* ... it only checks first program stream (->programs[0]) for data */
		for(unsigned int i = 0; i < av_format_context->programs[0]->nb_stream_indexes; i++ ){
			if( (av_format_context->streams[i] == NULL) ||
				(av_format_context->streams[i]->codec == NULL) )
				continue;
			switch( av_format_context->streams[i]->codec->codec_type ){
			case AVMEDIA_TYPE_VIDEO:
				ROSE_PRINT(0x4, "Found video stream (index = %d).", i);
				SetVideoStreamInfo( av_format_context, i, media_info_context );
				break;
			case AVMEDIA_TYPE_AUDIO:
				ROSE_PRINT(0x4, "Found audio stream (index = %d).", i);
				SetAudioStreamInfo( av_format_context, i, media_info_context );
				if( av_format_context->streams[i]->codec->codec_id != CODEC_ID_AAC )
					continue;
				if( (media_info_context->container_info.nId == Multimedia::CONTAINER_MPEGTS) || 
						(media_info_context->container_info.nId == Multimedia::CONTAINER_MPEGTTS) ){
					if( av_format_context->streams[i]->codec->sub_id != 1000 )
						continue;
					ROSE_PRINT(0x4, "THIS IS   __ HE-AAC__");
					media_info_context->stream_info[i].audio_info.AudioCodecID = Multimedia::AUDIO_HEAAC; // 0x0D;
				} else {
					if( av_format_context->streams[i]->codec->extradata == NULL )
						continue;
					media_info_context->stream_info[i].audio_info.nSubId = getMpeg4AACObjetType222( 
						av_format_context->streams[i]->codec->extradata, av_format_context->streams[i]->codec->extradata_size );
					if ( av_format_context->streams[i]->codec->codec_tag != 0x6134706D ) /* mp41 */
						continue;
					if( getMpeg4AACSBR222(av_format_context->streams[i]->codec->extradata,
							av_format_context->streams[i]->codec->extradata_size) == 1 ){
						ROSE_PRINT(0x4, "THIS IS   __ HE-AAC__");
						media_info_context->stream_info[i].audio_info.AudioCodecID = Multimedia::AUDIO_HEAAC; // 0x0D;
					}
				}
				break;
			case AVMEDIA_TYPE_SUBTITLE:
				ROSE_PRINT(0x4, "Found subtitle stream (index = %d).", i);
				SetSubtitleStreamInfo( av_format_context, i, media_info_context );
				break;
			default:
				ROSE_PRINT(0x4, "Unhandled TS index = %i, codec_type = %i", i, av_format_context->streams[i]->codec->codec_type );
				break;
			}
		}
#else
		for(unsigned int j = 0; j < av_format_context->nb_programs; j++ ){
			/* AVMetadataTag *name = av_metadata_get(ic->programs[j]->metadata, "name", NULL, 0);
			PRINTF("Program [%d]: %s\n", av_format_context->programs[j]->id, name ? name->value : "<empty>"); */
			for(unsigned int k = 0; k < av_format_context->programs[j]->nb_stream_indexes; k++ ){
				int i = av_format_context->programs[j]->stream_index[k];
				PRINTF("Program ID [%d] Stream [%d] Index [%d]", av_format_context->programs[j]->id, k, i ); 
				if( (av_format_context->streams[i] == NULL) ||
					(av_format_context->streams[i]->codec == NULL) )
					continue;
				switch( av_format_context->streams[i]->codec->codec_type ){
				case AVMEDIA_TYPE_VIDEO:
					ROSE_PRINT(0x4, "Found video stream (index = %d).", i);
					SetVideoStreamInfo( av_format_context, i, media_info_context );
					break;
				case AVMEDIA_TYPE_AUDIO:
					ROSE_PRINT(0x4, "Found audio stream (index = %d).", i);
					SetAudioStreamInfo( av_format_context, i, media_info_context );
					if( av_format_context->streams[i]->codec->codec_id != CODEC_ID_AAC )
						continue;
					if( (media_info_context->container_info.nId == Multimedia::CONTAINER_MPEGTS) || 
							(media_info_context->container_info.nId == Multimedia::CONTAINER_MPEGTTS) ){
						if( av_format_context->streams[i]->codec->sub_id != 1000 )
							continue;
						ROSE_PRINT(0x4, "THIS IS   __ HE-AAC__");
						media_info_context->stream_info[i].audio_info.AudioCodecID = Multimedia::AUDIO_HEAAC; // 0x0D;
					} else {
						if( av_format_context->streams[i]->codec->extradata == NULL )
							continue;
						media_info_context->stream_info[i].audio_info.nSubId = getMpeg4AACObjetType222( 
							av_format_context->streams[i]->codec->extradata, av_format_context->streams[i]->codec->extradata_size );
						if ( av_format_context->streams[i]->codec->codec_tag != 0x6134706D ) /* mp41 */
							continue;
						if( getMpeg4AACSBR222(av_format_context->streams[i]->codec->extradata,
								av_format_context->streams[i]->codec->extradata_size) == 1 ){
							ROSE_PRINT(0x4, "THIS IS   __ HE-AAC__");
							media_info_context->stream_info[i].audio_info.AudioCodecID = Multimedia::AUDIO_HEAAC; // 0x0D;
						}
					}
					break;
				case AVMEDIA_TYPE_SUBTITLE:
					ROSE_PRINT(0x4, "Found subtitle stream (index = %d).", i);
					SetSubtitleStreamInfo( av_format_context, i, media_info_context );
					break;
				default:
					ROSE_PRINT(0x4, "Unhandled TS index = %i, codec_type = %i", i, av_format_context->streams[i]->codec->codec_type );
					break;
				}
			}
		}
#endif

	} else {
		/* loc_FBB680 */
		ROSE_PRINT(0x4, "Processing streams...");
		for(unsigned int i = 0; i < av_format_context->nb_streams; i++ ){
			switch( av_format_context->streams[i]->codec->codec_type ){
			case AVMEDIA_TYPE_VIDEO:
				SetVideoStreamInfo( av_format_context, i, media_info_context );
				break;
			case AVMEDIA_TYPE_AUDIO:
				SetAudioStreamInfo( av_format_context, i, media_info_context );
				if( av_format_context->streams[i]->codec->codec_id != CODEC_ID_AAC )
					continue;
				if( (media_info_context->container_info.nId == Multimedia::CONTAINER_MPEGTS) || 
						(media_info_context->container_info.nId == Multimedia::CONTAINER_MPEGTTS) ){
					if( av_format_context->streams[i]->codec->sub_id != 1000 )
						continue;
					ROSE_PRINT(0x4, "THIS IS   __ HE-AAC__");
					media_info_context->stream_info[i].audio_info.AudioCodecID = Multimedia::AUDIO_HEAAC; // 0x0D;
				} else {
					if( av_format_context->streams[i]->codec->extradata == NULL )
						continue;
					media_info_context->stream_info[i].audio_info.nSubId = getMpeg4AACObjetType222( 
						av_format_context->streams[i]->codec->extradata, av_format_context->streams[i]->codec->extradata_size );
					if ( av_format_context->streams[i]->codec->codec_tag != 0x6134706D ) /* mp41 */
						continue;
					if( getMpeg4AACSBR222(av_format_context->streams[i]->codec->extradata,
							av_format_context->streams[i]->codec->extradata_size) == 1 ){
						ROSE_PRINT(0x4, "THIS IS   __ HE-AAC__");
						media_info_context->stream_info[i].audio_info.AudioCodecID = Multimedia::AUDIO_HEAAC; // 0x0D;
					}
				}
				break;
			case AVMEDIA_TYPE_SUBTITLE:
				SetSubtitleStreamInfo( av_format_context, i, media_info_context );
				break;
			default:
				ROSE_PRINT(0x4, "Unhandled stream index = %i, codec_type = %i", i, av_format_context->streams[i]->codec->codec_type );
				break;
			}
		}
	}

	/* Lets try to fix=>hide invalid audio streams */
	for( int i = 0; i < MAX_STREAMS; i++ ){
		switch( media_info_context->stream_info[i].stream_type ){
		case Multimedia::STREAM_TYPE_AUDIO:
			if( media_info_context->stream_info[i].audio_info.nSampleRate == 0 ){
				ROSE_PRINT(0x4, "Stream index = %i (AUDIO) has samplerate=0. Removing stream.", i );
				media_info_context->stream_info[i].stream_type = Multimedia::STREAM_TYPE_NONE;
				media_info_context->stream_info[i].stream_id   = 0;
				media_info_context->nAudioStream--;
				continue;
			}
/*
			if( media_info_context->stream_info[i].audio_info.AudioCodecID == Multimedia::AUDIO_DTS ){
				ROSE_PRINT(0x4, "Stream index = %i (AUDIO) is not supported (AUDIO_DTS).", i );
				media_info_context->stream_info[i].stream_type = Multimedia::STREAM_TYPE_NONE;
				media_info_context->stream_info[i].stream_id   = 0;
				media_info_context->nAudioStream--;
				continue;
			}
*/
			break;
		default:
			break;
		}
	}

	av_close_input_file222( av_format_context );
	ROSE_PRINT(0x4,"[CParsingMedia] %s Out", "GetMediaTypeInfo" );
	return 1;
}
