/*
 * CParsingMedia.hpp
 * Copyright (C) 2010 Andreas Fiedler <smartsmurf@users.sourceforge.net>
 *
 * This file is part of samygodca, a free DTS Coherent Acoustics
 * stream decoder plugin for SamyGO extension of Samsung DTV.
 *
 * samygodca is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * samygodca is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef __CPARSINGMEDIA_H__
#define __CPARSINGMEDIA_H__

#include "Multimedia.hpp"
#include "Config.h"

/* FFMPEG defines and imports */
// #define FFMPEG_LIB_PATH "/mtd_rwarea/lib/"
typedef void (*av_register_all_t) (void);
typedef int (*av_open_input_file_t)(AVFormatContext **, const char *,AVInputFormat *,int, AVFormatParameters *);
typedef void (*av_close_input_file_t)(AVFormatContext *);
typedef int (*av_find_stream_info_t)(AVFormatContext *);
typedef void (*dump_format_t)(AVFormatContext *ic, int index, const char *url, int is_output);

typedef int (*getMpeg4AACObjetType_t)(const uint8_t *buf, int buf_size);
typedef int (*getMpeg4AACSBR_t)(const uint8_t *buf, int buf_size);

/* FFMPEG/LIBDCA module imports */
extern "C" DLL_LOCAL char path_ext_ffmpeg[];

extern av_register_all_t av_register_all222;
extern av_open_input_file_t av_open_input_file222;
extern av_close_input_file_t av_close_input_file222;
extern av_find_stream_info_t av_find_stream_info222;
extern dump_format_t dump_format222;
extern getMpeg4AACObjetType_t getMpeg4AACObjetType;
extern getMpeg4AACSBR_t getMpeg4AACSBR;

/* ffmpeg library interface */
struct ffmpeg_interface_t {
	int  (*av_open_input_file)(AVFormatContext **ic_ptr, const char *filename,
                       AVInputFormat *fmt,
                       int buf_size,
                       AVFormatParameters *ap);
	int  (*av_close_input_file)(AVFormatContext *s);
	int  (*av_find_stream_info)(AVFormatContext *ic);
	AVCodec * (*avcodec_find_decoder)(enum CodecID id);
	int  (*avcodec_open)(AVCodecContext *avctx, AVCodec *codec);
	int  (*avcodec_close)(AVCodecContext *avctx);
	int  (*av_read_frame)(AVFormatContext *s, AVPacket *pkt);
	int  (*avcodec_decode_video2)(AVCodecContext *avctx, AVFrame *picture,
                         int *got_picture_ptr,
                         AVPacket *avpkt); 
	int  (*avcodec_decode_audio3)(AVCodecContext *avctx, int16_t *samples,
                         int *frame_size_ptr,
                         AVPacket *avpkt);
	void * (*av_malloc)(unsigned int size);
	void   (*av_free)(void *ptr);
	AVFrame * (*avcodec_alloc_frame)(void);
	int  (*av_dup_packet)(AVPacket *pkt);
	int  (*avcodec_decode_subtitle2)(AVCodecContext *avctx, AVSubtitle *sub,
                            int *got_sub_ptr,
                            AVPacket *avpkt);
	void  (*av_free_packet)(AVPacket *pkt);
	ReSampleContext * (*av_audio_resample_init)(int output_channels, int input_channels,
                                        int output_rate, int input_rate,
                                        enum SampleFormat sample_fmt_out,
                                        enum SampleFormat sample_fmt_in,
                                        int filter_length, int log2_phase_count,
                                        int linear, double cutoff);
	int  (*audio_resample)(ReSampleContext *s, short *output, short *input, int nb_samples);
	int  (*audio_resample_close)(ReSampleContext *s);
	int  (*av_seek_frame)(AVFormatContext *s, int stream_index, int64_t timestamp,
                  int flags);
	int  (*avcodec_flush_buffers)(AVCodecContext *avctx); 
	int  (*av_WV_setSpeed)();
	int  (*av_WV_GetAvailableBitrates)();
	int  (*av_WV_GetCurrentBitrates)();
	int  (*av_WV_GetErrorCode)();
	int  (*av_WV_GetCopyProtection)();
	int  (*av_WV_GetTotalTime)();
	int  (*av_WV_GetCurrentTime)();
	int  (*av_WV_GetMediaTime)();
	int  (*av_read_frame_flush)();
	int  (*av_WMDRM_setLicence)();
	int  (*av_NFX_SetCallback4WMDRM)();
	int  (*av_NFX_Get_Log_Info)();
	int  (*av_NFX_Get_CopyProtectInfo)();
	int  (*av_NFX_Get_Url_Info)();
	int  (*av_VUDU_SetCurrentPosition)();
	AVMetadataTag * (*av_metadata_get)(AVMetadata *m, const char *key, const AVMetadataTag *prev, int flags);
	int  (*av_seek_available)();
	void  (*url_set_interrupt_cb)(URLInterruptCB *interrupt_cb); 
	void  (*av_freep)(void *ptr);
	int  initialised;
	int  (*av_register_all)(void);
	void (*avcodec_register_all)(void);
	void (*av_init_packet)(AVPacket *);
	AVCodecContext* (*avcodec_alloc_context)(void);
	int (*avcodec_encode_audio)(AVCodecContext *avctx, uint8_t *buf, int buf_size, const short *samples);
	AVCodec* (*avcodec_find_encoder)(enum CodecID); 
};
extern struct ffmpeg_interface_t ffmpeg;

class LIB_EXPORT CParsingMedia
{
public:
	static void * hLib_avutil;
	static void * hLib_avcodec;
	static void * hLib_avformat;
	static char * dlError;
	static int m_bFirst;

	/* CParsingMedia(void); */
	/* virtual ~CParsingMedia(void); */
	void SetAudioStreamInfo(AVFormatContext * ffMediaContext, unsigned int astream_index, Multimedia::_tMediaInfoContext * dtvMediaContext);
	void SetSubtitleStreamInfo(AVFormatContext * ffMediaContext, unsigned int stream_index, Multimedia::_tMediaInfoContext * media_info_context);
	void SetVideoStreamInfo(AVFormatContext * ffMediaContext, unsigned int stream_index, Multimedia::_tMediaInfoContext * media_info_context);
	void SetContainerInfo(AVFormatContext * ffMediaContext, char const * stream_name, Multimedia::_tMediaInfoContext * media_info_context);
	int AnalyseTSstream(char  const*);
	int GetMediaTypeInfo(char  const* file_name, Multimedia::_tMediaInfoContext * media_info_context );
	void PrepareParsing(void);
	void PrepareParsingExtFfmpeg(void);
	void LoadImportsFfmpeg( struct ffmpeg_interface_t * ff );
};

#endif
