/*
 * Logging.cpp
 * Copyright (C) 2010 Andreas Fiedler <smartsmurf@users.sourceforge.net>
 *
 * This file is part of samygodca, a free DTS Coherent Acoustics
 * stream decoder plugin for SamyGO extension of Samsung DTV.
 *
 * samygodca is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * samygodca is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <dirent.h>
#include "Logging.h"

#define TESTFILE "samygo_testfile"
#define LOGBUFSIZE 1024

static int pvr_logfd = -1;

int
pvr_log(const char *fmt, ...)
{
	va_list ap;
	char buf[LOGBUFSIZE];
	ssize_t written;
	int len;
	
	if (pvr_logfd < 0)
		return (0);

	va_start(ap, fmt);
	len = vsnprintf(buf, sizeof(buf), fmt, ap);
	va_end(ap);
	if (len < 0)
		return (-1);
	if (len > (int)sizeof(buf) - 1)
		len = sizeof(buf) - 1;
	written = write(pvr_logfd, buf, len);
	if (written != len)
		return (-1);

	return 0;
}

int
pvr_log_init(const char *fname)
{
	int fd;

	if ((fd = open(fname, O_WRONLY|O_APPEND|O_CREAT|O_SYNC|O_TRUNC, 0644)) >= 0) {
		pvr_log_deinit();
		pvr_logfd = fd;

		return (0);
	}
	return (-1);
}

int
pvr_log_stdout(void)
{
	if (pvr_logfd < 0)
		return (-1);
	if (dup2(pvr_logfd, STDOUT_FILENO) < 0) {
		LOG("dup2() failed: %s", strerror(errno));
		return (-1);
	}

	return (0);
}

int
pvr_log_stderr(void)
{
	if (pvr_logfd < 0)
		return (-1);
	if (dup2(pvr_logfd, STDERR_FILENO) < 0) {
		LOG("dup2() failed: %s", strerror(errno));
		return (-1);
	}

	return (0);
}

int
pvr_log_deinit(void)
{
	if (pvr_logfd >= 0) {
		close(pvr_logfd);
		pvr_logfd = -1;
	}
	return (0);
}

/*
int
strlcpy(char *dst, const char *src, size_t dst_size)
{
	if (dst_size == 0)
		return (0);
	strncpy(dst, src, dst_size);
	dst[dst_size - 1] = '\0';

	return (0);
}

int
strlcat(char *dst, const char *src, size_t dst_size)
{
	if (dst_size == 0)
		return (0);
	strncat(dst, src, dst_size);
	dst[dst_size - 1] = '\0';

	return (0);
}
*/

int pvr_log_hex( const unsigned char * buffer, int buffsize )
{
	int i;
	char tmpbuf[8];
	for(i = 0; i < buffsize; i++ ){
		if( i && ((i & 15) == 0) )
			pvr_log( "\n" );
		sprintf( tmpbuf, "%02X ", *buffer++ );
		pvr_log( tmpbuf );	
	}
	pvr_log( "\n" );

	return 0;
}
