/*
 * Logging.h
 * Copyright (C) 2010 Andreas Fiedler <smartsmurf@users.sourceforge.net>
 *
 * This file is part of samygodca, a free DTS Coherent Acoustics
 * stream decoder plugin for SamyGO extension of Samsung DTV.
 *
 * samygodca is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * samygodca is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef __LOGGING_H__
#define __LOGGING_H__

#include "Config.h"

#define ROSE_PRINT(level, fmt, ...)	\
	pvr_log( fmt "\n", ##__VA_ARGS__)

#define SD_PRINT(level, fmt, ...) \
	if( gSdPrintMask & level ) \
	    PRINTF(fmt, ##__VA_ARGS__)

#define PRINTF(fmt, ...)	\
	pvr_log( fmt "\n", ##__VA_ARGS__)

#define PRINTHEX( infoline, buffer, buffsize )	\
	pvr_log(infoline "\n"); pvr_log_hex( buffer, buffsize );

#define LOG(fmt, ...)											\
	pvr_log("%s:%d:" fmt "\n", __PRETTY_FUNCTION__, __LINE__,	\
			##__VA_ARGS__)

#ifdef __DEBUG__
 #define spI_CheckPrintLevel( level ) \
 	((const unsigned int)g_u32PrintLevel & level)

 #define spI_CheckPrintIsr() \
	(g_u32PrintLevel != 0)

#else
 #define spI_CheckPrintLevel(level) (0)
 #define spI_CheckPrintIsr() (0)
#endif

#ifdef __cplusplus
extern "C" {
#endif

DLL_LOCAL int pvr_log( const char *fmt, ... ) __attribute__ ((noinline))
	__attribute__((__format__(printf, 1, 2)));
DLL_LOCAL int pvr_log_init(const char * fname);
DLL_LOCAL int pvr_log_deinit(void);
DLL_LOCAL int pvr_log_stdout(void); 
DLL_LOCAL int pvr_log_stderr(void); 
DLL_LOCAL int pvr_log_hex(const unsigned char * , int);

/* exeDSP spI logging */
extern unsigned int g_u32PrintLevel;
extern unsigned int gSdPrintMask;

#ifdef __cplusplus
}
#endif

#endif
