/*
 * Multimedia.hpp
 * Copyright (C) 2010 Andreas Fiedler <smartsmurf@users.sourceforge.net>
 *
 * This file is part of samygodca, a free DTS Coherent Acoustics
 * stream decoder plugin for SamyGO extension of Samsung DTV.
 *
 * samygodca is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * samygodca is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef __MULTIMEDIA_HPP__
#define __MULTIMEDIA_HPP__

#define __STDC_CONSTANT_MACROS 1
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include "libavformat/avformat.h"
#include "Config.h"
#include "MediaCommon.hpp"
#include "src/spITsdComm.h"

enum SdContainerFormat_k {
	SDCONTAINERFORMAT_NONE		= 0x00,
	SDCONTAINERFORMAT_01		= 0x01,
	SDCONTAINERFORMAT_02		= 0x02,
	SDCONTAINERFORMAT_MPEG		= 0x03,
	SDCONTAINERFORMAT_MPEGTS	= 0x04,
	SDCONTAINERFORMAT_ASF		= 0x05,
	SDCONTAINERFORMAT_06		= 0x06,
	SDCONTAINERFORMAT_07		= 0x07,
	SDCONTAINERFORMAT_AVI		= 0x08,
	SDCONTAINERFORMAT_09		= 0x09,
	SDCONTAINERFORMAT_MP3		= 0x0A,
	SDCONTAINERFORMAT_MP4		= 0x0B,
	SDCONTAINERFORMAT_12		= 0x0C,
	SDCONTAINERFORMAT_13		= 0x0D,
	SDCONTAINERFORMAT_14		= 0x0E,
	SDCONTAINERFORMAT_MKV		= 0x0F,
	SDCONTAINERFORMAT_16		= 0x10,
	SDCONTAINERFORMAT_17		= 0x11,
	SDCONTAINERFORMAT_18		= 0x12,
	SDCONTAINERFORMAT_19		= 0x13,
	SDCONTAINERFORMAT_20		= 0x14
};

enum SdVideoFormat_k {
	SDVIDEOFORMAT_NONE	 = 0x00,
	SDVIDEOFORMAT_MPEG1  = 0x01,
	SDVIDEOFORMAT_MPEG2  = 0x02,
	SDVIDEOFORMAT_MPEG4  = 0x03,
	SDVIDEOFORMAT_H264   = 0x04,
	SDVIDEOFORMAT_DIVX   = 0x05,
	SDVIDEOFORMAT_WMV9   = 0x06,	/* WMV9 / VC1 */
	SDVIDEOFORMAT_XVID   = 0x07,
	SDVIDEOFORMAT_MJPEG  = 0x08,
};

enum SdAudioFormat_k {
	SDAUDIOFORMAT_NONE	= 0x00,
	SDAUDIOFORMAT_PCM	= 0x01,
	SDAUDIOFORMAT_ADPCM	= 0x02,
	SDAUDIOFORMAT_MP3	= 0x03,
	SDAUDIOFORMAT_4		= 0x04,
	SDAUDIOFORMAT_5		= 0x05,
	SDAUDIOFORMAT_AAC	= 0x06,
	SDAUDIOFORMAT_AC3	= 0x07,
	SDAUDIOFORMAT_HEAAC	= 0x08,
	SDAUDIOFORMAT_EAC3	= 0x09,
	SDAUDIOFORMAT_DTS	= 0x0A,
	SDAUDIOFORMAT_11	= 0x0B,
	SDAUDIOFORMAT_WMA	= 0x0C,
	SDAUDIOFORMAT_13	= 0x0D,
	SDAUDIOFORMAT_14	= 0x0E,
	SDAUDIOFORMAT_OTHERS= 0x0F,
	SDAUDIOFORMAT_16	= 0x10,
	SDAUDIOFORMAT_VORBIS= 0x11,
};

enum AvfdAudFormat_k {
	AVFDAUDFORMAT_MP3   = 0x00,
	AVFDAUDFORMAT_ADPCM = 0x01,
	AVFDAUDFORMAT_PCM	= 0x02,
	AVFDAUDFORMAT_03    = 0x03,
	AVFDAUDFORMAT_AC3	= 0x04,
	AVFDAUDFORMAT_WMA   = 0x05,
	AVFDAUDFORMAT_AAC   = 0x06,
/*	AVFDAUDFORMAT_07    = 0x07, */
	AVFDAUDFORMAT_DTS   = 0x07,
	AVFDAUDFORMAT_SKIP  = 0x08
};

enum ModuleState_k {
	MODULESTATE_NONE = 0x00,
	MODULESTATE_PLAY = 0x01,
	MODULESTATE_02 = 0x02,
	MODULESTATE_03 = 0x03,
};

enum SdFeeder_TransferFlag_k {
	SDFEEDER_TRANSFER_NONE = 0x00,
};

enum SdFeeder_CallbackType_k {
	SDFEEDER_CALLBACK_NONE = 0x00,
};

struct SdFeeder_Status_t {
	int Dummy;
};

struct esvideo_t {
	int	hResolution;
	int vResolution;
	int frameRate;
};

struct SdFeeder_Settings_t {
	char * pFilePath;				// 0x000
	SdContainerFormat_k eContainerType;		// 0x004
	SdVideoFormat_k eVidFormat;	// 0x008
	SdAudioFormat_k eAudFormat;	// 0x00C
	int videoStreamId;			// 0x010
	struct esvideo_t esvideo;	// 0x014...0x01C
	int audioStreamId;			// 0x020
	int captionStreamId;		// 0x024
	int speed;					// 0x028
	bool bReversePlay;			// 0x02C
	struct pDemuxInfoHandle_t * dmxHandle;		// 0x030
	int eAVDecId;				// 0x034
	int totalTime;				// 0x038
	int mRes03C;
	int64_t totalSize;			// 0x040
};

struct SdFeeder_CallBackParams_t {
	int Dummy;
};

namespace Multimedia
{
// public:
	enum _eStreamType {
		STREAM_TYPE_NONE     = 0x00,
		STREAM_TYPE_VIDEO    = 0x02,
		STREAM_TYPE_AUDIO    = 0x03,
		STREAM_TYPE_SUBTITLE = 0x04,
	};

	enum ContainerType {
		CONTAINER_NONE		= 0x00,
		CONTAINER_1			= 0x01,
		CONTAINER_2			= 0x02,
		CONTAINER_3			= 0x03,
		CONTAINER_MPEG		= 0x04,
		CONTAINER_MPEGTS	= 0x05,
		CONTAINER_ASF		= 0x06,
		CONTAINER_7			= 0x07,
		CONTAINER_8			= 0x08,
		CONTAINER_AVI		= 0x09,
		CONTAINER_10		= 0x0A,
		CONTAINER_MP3		= 0x0B,
		CONTAINER_MP4		= 0x0C,	// mov,mp4,m4a,3gp,3g2,mj2
		CONTAINER_MPEGTTS	= 0x0D,
		CONTAINER_MKV		= 0x0E,
		CONTAINER_15		= 0x0F,
	};

	enum AudioCodecType {
		AUDIO_NONE		= 0x00,
		AUDIO_01,
		AUDIO_OTHERS	= 0x02,
		AUDIO_MP3		= 0x03,
		AUDIO_04,
		AUDIO_AAC		= 0x05,
		AUDIO_AC3		= 0x06,
		AUDIO_DTS		= 0x07,
		AUDIO_PCM_S16	= 0x08,
		AUDIO_XLAW		= 0x09,
		AUDIO_WMA		= 0x0A,
		AUDIO_ADPCM		= 0x0B,
		AUDIO_EAC3		= 0x0C,
		AUDIO_HEAAC		= 0x0D,
		AUDIO_VORBIS	= 0x0E,
	};

	enum VideoCodecType {
		VIDEO_NONE		= 0x00,
		VIDEO_UNSUPPORTED = 0x01,
		VIDEO_MSMPEGV	= 0x02,
		VIDEO_MSMPEGV3	= 0x03,
		VIDEO_MPEG1		= 0x04,
		VIDEO_MPEG2		= 0x05,
		VIDEO_MPEG4		= 0x06,
		VIDEO_H264		= 0x07,
		VIDEO_VC1		= 0x08,
		VIDEO_WMV		= 0x09,
		VIDEO_WMV9		= 0x0A,
		VIDEO_DIVX		= 0x0B,
		VIDEO_DX50		= 0x0C,
		VIDEO_XVID		= 0x0D,
		VIDEO_14,
		VIDEO_MJPEG     = 0x0F,
		VIDEO_MJPEGB    = 0x10,
	};

	struct VIDEO_INFO {
		VideoCodecType VideoCodecID;		// (index*0x90)+0x0240
		int nPid;				// (index*0x90)+0x0244
		int64_t nTotalPlayTime; // (index*0x90)+0x0248
		int64_t nFrameCount;	// (index*0x90)+0x0250
		float nFrameRate;		// (index*0x90)+0x0258
		int nFrameWidth;		// (index*0x90)+0x025C
		int nFrameHeight;		// (index*0x90)+0x0260
		int mRes3[0x08];
		int nSubId;				// (index*0x90)+0x0284
	};

	struct AUDIO_INFO {
		AudioCodecType AudioCodecID;		// (index*0x90)+0x0288
		int nPid;				// (index*0x90)+0x028C
		int64_t nTotalPlayTime; // (index*0x90)+0x0290
		int mRes2;
		int nSampleRate;		// (index*0x90)+0x029C
		int nBitRate;			// (index*0x90)+0x02A0
		int nRes3;				// (index*0x90)+0x02A4
		int nChannel;			// (index*0x90)+0x02A8
		int nSubId;				// (index*0x90)+0x02AC
		int nBlockAlign;		// (index*0x90)+0x02B0
		int mRes4;				// (index*0x90)+0x02B4
		int mRes5;				// (index*0x90)+0x02B8
		short nCodecTag;		// (index*0x90)+0x02BC
		short nValidBitsPerSample;		// (index*0x90)+0x02BE
	};

	struct _tSubtitleInfo {
		int nSubtitleFlag;		// (index*0x90)+0x02C0
		char cSubtitleLang[4];	// (index*0x90)+0x02C4
	};

	struct _tStreamInfoContext {
		_eStreamType stream_type;						// (index*0x90)+0x0238
		int	stream_id;							// (index*0x90)+0x023C
		struct VIDEO_INFO video_info;			// (index*0x90)+0x0240...0x0287
		struct AUDIO_INFO audio_info;			// (index*0x90)+0x0288...0x02BF
		struct _tSubtitleInfo subtitle_info;	// (index*0x90)+0x02C0
	};

	struct _tFileInfo {
		char filename[512];												// 0x0000
		int64_t filesize;												// 0x0200
	};

	struct _tContainerInfo {
		ContainerType	nId;									// 0x0208
		int  mRes1;														// 0x020C
		int64_t nTotalPlayTime;											// 0x0210
		int  mRes2[2];
	};

	struct _tMediaInfoContext {
		_tFileInfo		file_info;										// 0x0000
		_tContainerInfo container_info;									// container_info.nID : 0x0208
		int	 nStreams;													// 0x0220
		int  nPcrPid;													// 0x0224
		int  nAudioStream;												// 0x0228
		int  nVideoStream;												// 0x022C
		int  nSubtitleStream;											// 0x0230
		int  mRes1;
		struct _tStreamInfoContext stream_info[MAX_STREAMS];				// 0x0238 (20*0x90 = 0xB40)
		int nBitRate;													// 0x0D78
	};
};


#endif
