/*
 * samygodca.c
 * Copyright (C) 2010 Andreas Fiedler <smartsmurf@users.sourceforge.net>
 *
 * This file is part of samygodca, a free DTS Coherent Acoustics
 * stream decoder plugin for SamyGO extension of Samsung DTV.
 *
 * samygodca is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * samygodca is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <dlfcn.h>
#include <sys/mman.h>

#define LOG_FILE "/dtv/samygodca.log"

/* ffmpeg lib handles */
/*
void * _ZN13CParsingMedia13hLib_avformatE;
void * _ZN13CParsingMedia12hLib_avcodecE;
void * _ZN13CParsingMedia11hLib_avutilE;
*/


/*
extern unsigned int gSdPrintMask;
extern unsigned int gSdTraceMask;
extern unsigned int _ZN10CRosePrint17m_uRosePrintLevelE;
extern unsigned int g_u32PrintLevel;
extern unsigned int g_u32PrintModule;
*/

static FILE * logfile = NULL;

/* injector main entry */
typedef void (*lib_samygodca_init)(const char *, const char * ); 

static void cacheflush(void *start, size_t size)
{
   /* For details, please see the Linux sources, files
    * arch/arm/kernel/traps.c, arch/arm/kernel/entry-common.S and
    * asm-arm/unistd.h
    *
    * This syscall is supported by all 2.4 and 2.6 Linux kernels.  2.2 linux
    * kernels got first support with version 2.2.18.
    */

   asm("mov r0, %0\n"
       "mov r1, %1\n"
       "mov r2, #0\n"
       /* EABI or Thumb syscall: syscall number passed in 'r7' (syscall base
	* number is 0x0).  Note that Thumb and EABI are generally not the
	* same.  It just happens that the simple cacheflush syscall doesn't
	* expose any of differences in calling conventions.
        */
       "mov r7, #0xf0000\n"
       "add r7, #0x02\n"
       "swi #0\n" ::
       "g"(start), "g"((long)start+size) :
       "r0", "r1", "r2", "r7");
}

/*
void patch_opcode( void * hExeDsp, const char * funcname, int offset, unsigned int old_opcode, unsigned int new_opcode )
{
	unsigned int exe_page;
	unsigned int dwOpCode;
	void * pfunc;

	pfunc = dlsym( hExeDsp, funcname );
	if( pfunc == NULL ){
		fprintf( logfile, "<patch_opcode>: unable to get export(1). Reason: %s.\n", dlerror() );
		return;
	}

	dwOpCode = *(unsigned long *)((unsigned int)pfunc+offset);
	fprintf( logfile, "<patch_opcode>: Current opcode = 0x%08X.\n", dwOpCode );
	if( dwOpCode == old_opcode ){
		fprintf( logfile, "<patch_opcode>: Patching exeDSP...\n" );
		exe_page = (unsigned int)pfunc  & ~0x0FFF;
		mprotect( (void *)exe_page, 8192, PROT_READ | PROT_WRITE | PROT_EXEC ); 
		*(unsigned long *)((unsigned int)pfunc+offset) = new_opcode;
		mprotect( (void *)exe_page, 8192, PROT_READ | PROT_EXEC ); 
		cacheflush((void *)((unsigned int)pfunc+offset), 4); 
	} else {
		fprintf( logfile, "<patch_opcode>: Patch already applied.\n" );
	}
}
*/

/*
 Patch code in exeDSP memory
 - verifies current memory before updating with patch code
 - opcode_offset and num_opcodes are denoted in 4-byte-words
 - old_opcodes might be NULL -> no check will be performed
*/
int
patch_opcodes( void * hExeDsp, char * funcname, int opcode_offset, const unsigned int * old_opcodes, const unsigned int * new_opcodes, int num_opcodes, int access_mode )
{
	unsigned int exe_page;
	int * pfunc = NULL;

	pfunc = dlsym( hExeDsp, funcname );
	if( pfunc == NULL ){
		fprintf( logfile, "<patch_opcode>: unable to get export(1). Reason: %s.\n", dlerror() );
		return (-1);
	}

	if( old_opcodes ){
		if( memcmp(&pfunc[opcode_offset], old_opcodes, 4*num_opcodes) ){
			fprintf( logfile, "Patch either already applied or different exeDSP code. Can't patch this.");
			return (-1);
		}
	}

	exe_page = (unsigned int)&pfunc[opcode_offset] & ~0xFFF;
	mprotect( (void *)exe_page, 8192, PROT_READ | PROT_WRITE | PROT_EXEC ); 
	memcpy( &pfunc[opcode_offset], new_opcodes, 4*num_opcodes);
	mprotect( (void *)exe_page, 8192, access_mode ); 
	cacheflush( &pfunc[opcode_offset], 4*num_opcodes); 
	fprintf( logfile, "Patched '%s' successfully.\n", funcname);
	return 0;
}


void * open_module( const char * mod_path, const int mod_flags )
{
	void * lib_handle = dlopen( mod_path, mod_flags );
	if( lib_handle )
		fprintf( logfile, "<Game_Main>: handle to '%s' = 0x%08X.\n", mod_path, (int)lib_handle );
	else
		fprintf( logfile, "<Game_Main>: unable to load '%s'. Reason: %s.\n", mod_path, dlerror() );
	return lib_handle;
}

int Game_Main(const char  *path, const char *udn __attribute__ ((unused)))
{
/*	char * ldlibPath; */
	char lib_buf[512];
	void * hExeDsp;
	void * hInjMod;
	void * lib_export;
	lib_samygodca_init samygodca_init;
	unsigned int old_opcodes[32];
	unsigned int new_opcodes[32];

	logfile = fopen( LOG_FILE, "a+" );
	fprintf( logfile, "<Game_Main>: module loaded.\n" );
	fprintf( logfile, "<Game_Main>: path = '%s'.\n", path );
	fflush( logfile );

	/* if ffmpeg libraries already loaded... */
	/* ...close them in order to load our libs */
/*
	if( _ZN16CMovieInfoParser8m_bFirstE ){
		dlclose( _ZN13CParsingMedia13hLib_avformatE ); _ZN13CParsingMedia13hLib_avformatE = NULL;
		dlclose( _ZN13CParsingMedia12hLib_avcodecE ); _ZN13CParsingMedia12hLib_avcodecE = NULL;
		dlclose( _ZN13CParsingMedia11hLib_avutilE ); _ZN13CParsingMedia11hLib_avutilE = NULL;
		_ZN16CMovieInfoParser8m_bFirstE = 0;
	}
*/

	hExeDsp = open_module( NULL, RTLD_LAZY | RTLD_GLOBAL);
	if( hExeDsp == NULL )
		goto game_main_close;

	/* load inject module */
	snprintf(lib_buf, sizeof(lib_buf), "%s%s", path, "samygodca_inj.so");
	hInjMod = dlopen( lib_buf, RTLD_NOW | RTLD_GLOBAL | RTLD_NODELETE );
	if( hInjMod == NULL ){
		fprintf( logfile, "<Game_Main>: unable to load inject module. Reason: %s.\n", dlerror() );
		goto game_main_close;
	}

	samygodca_init = (lib_samygodca_init)dlsym( hInjMod, "samygodca_init");
	if( samygodca_init )
		samygodca_init(path, "/dtv/samygodca_inj.log");

	fprintf( logfile, "Patching CheckSupportMovie()...\n");
	lib_export = dlsym( hInjMod, "CheckSupportMovie" );
	if( lib_export == NULL )
		goto game_main_close;
	new_opcodes[0] = 0xE51FF004;
	new_opcodes[1] = (int)lib_export;
	patch_opcodes( hExeDsp, "_Z17CheckSupportMovieN10CMovieTDIF13MovieSettingsE", 0x00, 
		NULL, new_opcodes, 2, PROT_READ | PROT_EXEC );

	/* insert hook for CParsingMedia::PrepareParsing(void) */
	fprintf( logfile, "Patching CParsingMedia_PrepareParsing()...\n");
	lib_export = dlsym( hInjMod, "CParsingMedia_PrepareParsing" );
	if( lib_export == NULL )
		goto game_main_close;
	new_opcodes[0] = 0xE51FF004;
	new_opcodes[1] = (int)lib_export;
	patch_opcodes( hExeDsp, "_ZN13CParsingMedia14PrepareParsingEv", 0x00, 
		NULL, new_opcodes, 2, PROT_READ | PROT_EXEC );

	/* insert hook for ParsingMedia::GetMediaTypeInfo(char  const* file_name, Multimedia::_tMediaInfoContext * media_info_context ) */
	fprintf( logfile, "Patching CParsingMedia_GetMediaTypeInfo()...\n");
	lib_export = dlsym( hInjMod, "CParsingMedia_GetMediaTypeInfo" );
	if( lib_export == NULL )
		goto game_main_close;
	new_opcodes[0] = 0xE51FF004;
	new_opcodes[1] = (int)lib_export;
	patch_opcodes( hExeDsp, "_ZN13CParsingMedia16GetMediaTypeInfoEPKcPN10Multimedia18_tMediaInfoContextE", 0x00, 
		NULL, new_opcodes, 2, PROT_READ | PROT_EXEC );

	/*
	2C 30 D0 E5                 LDRB    R3, [R0,#0x2C]
	->
	2C 30 C0 E5                 STRB    R3, [R0,#0x2C]
	*/
	// patch_opcode( hExeDsp, "_ZN14TDsChelseaMpeg18StartAudioDecodingEv", 0x00, 0xE5D0302C, 0xE5C0302C );

	/* insert hook for CMovieCore::SetMediaInfo */
	/* the new function maps DTS to PCM settings */
	fprintf( logfile, "Patching CMovieCore::SetMediaInfo()...\n");
	lib_export = dlsym( hInjMod, "CMovieCore_SetMediaInfo" );
	if( lib_export ){
		new_opcodes[0] = 0xE51FF004;
		new_opcodes[1] = (int)lib_export;
		patch_opcodes( hExeDsp, "_ZN10CMovieCore12SetMediaInfoEPN10Multimedia18_tMediaInfoContextE", 0x00, 
			NULL, new_opcodes, 2, PROT_READ | PROT_EXEC );
	}

	/* insert hook for CMovieCore::SelectAudio */
	/* the new function maps DTS to PCM settings */
	fprintf( logfile, "Patching CMovieCore::SelectAudio()...\n");
	lib_export = dlsym( hInjMod, "CMovieCore_SelectAudio" );
	if( lib_export ){
		new_opcodes[0] = 0xE51FF004;
		new_opcodes[1] = (int)lib_export;
		patch_opcodes( hExeDsp, "_ZN10CMovieCore11SelectAudioEPN10Multimedia18_tMediaInfoContextEj", 0x00, 
			NULL, new_opcodes, 2, PROT_READ | PROT_EXEC );
	}

	/* modify avfd interfacer functions for audio_codec with AVFDAUDFORMAT_PCM (2) */
	lib_export = dlsym( hExeDsp, "gAvfdInterfaceAudioCodecFunctions" );
	if( lib_export == NULL )
		goto game_main_close;
	memcpy( old_opcodes, lib_export+ 4*2*25, 4*25 );
	memcpy( new_opcodes, lib_export+ 4*2*25, 4*25 );

	lib_export = dlsym( hInjMod, "spIAe_Open_DTS" );
	if( lib_export )
		new_opcodes[0] = (int)lib_export;

	lib_export = dlsym( hInjMod, "spIAe_Close_DTS" );
	if( lib_export )
		new_opcodes[1] = (int)lib_export;

	lib_export = dlsym( hInjMod, "InterfacerAudioTransferData_DTS" );
	if( lib_export )
		new_opcodes[2] = (int)lib_export;

	lib_export = dlsym( hInjMod, "InterfacerAudioTransferDataWithSyncHdr_DTS" );
	if( lib_export )
		new_opcodes[3] = (int)lib_export;

	lib_export = dlsym( hInjMod, "InterfacerAudioTransferBufDataWithSyncHdr_DTS" );
	if( lib_export )
		new_opcodes[4] = (int)lib_export;
	
	lib_export = dlsym( hInjMod, "InterfacerContainerAeCopyData_DTS" );
	if( lib_export )
		new_opcodes[18] = (int)lib_export;

	patch_opcodes( hExeDsp, "gAvfdInterfaceAudioCodecFunctions", 2*25, old_opcodes, new_opcodes, 25, PROT_READ | PROT_WRITE );

	/* some debugging stuff */
	// _ZN10CRosePrint17m_uRosePrintLevelE |= 0x4004;
	// gSdTraceMask |= 0x24;
	/* g_u32PrintLevel &= ~0x200000; */
	/* g_u32PrintLevel |= 0x200000; */

	/* avfd debug logging */
	// gSdPrintMask |= 0x800;
	// g_u32PrintModule |= 0x80000000;

	dlclose(hExeDsp);

game_main_close:
	fclose( logfile );
	return 1;
}
