/*
 * ffmpeg-dcadec.c
 * Copyright (C) 2010 Andreas Fiedler <smartsmurf@users.sourceforge.net>
 *
 * simple dts -> pcm decoder, based on sample from ffmpeg libavcodec/api-example.c
 *
 * This file is part of samygodca, a free DTS Coherent Acoustics
 * stream decoder plugin for SamyGO extension of Samsung DTV.
 *
 * samygodca is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * samygodca is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <limits.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include <libavcodec/avcodec.h>
#include <libavformat/avformat.h>
#include <libavutil/mathematics.h>

void dump_hex( unsigned char * pbData, int data_len)
{
	int i;
	for( i=0; i<data_len; i++ ){
	   if( i && ((i & 15) == 0) )
		   printf("\n");
	   printf("%2.2X ", pbData[i]);
	}
	printf("\n");
}

int demux_audio_example(const char *filename, const char *outfilename)
{
	int i, audioStream;
	int sample_count;
    AVFormatContext *pFormatCtx;
    AVCodecContext  *pCodecCtx;
    AVCodec         *pCodec;
    AVCodecContext  *pCodecCtxPCM;
    AVCodec         *pCodecPCM;
	AVFrame         *pFrame;
    AVPacket        packet; 
	/* unsigned char	outbuf[AVCODEC_MAX_AUDIO_FRAME_SIZE]; */
	unsigned char   pcmbuf[AVCODEC_MAX_AUDIO_FRAME_SIZE];
	short samples[AVCODEC_MAX_AUDIO_FRAME_SIZE/sizeof(short)];
	int frame_size;
	int out_size, len;
	FILE * outfile = NULL;

	if( !outfilename )
		return 0;

	outfile = fopen( outfilename, "wb" );

	if( !outfile )
		return 0;

    /* find the requested audio encoder */
    pCodecPCM = avcodec_find_encoder(CODEC_ID_PCM_S16BE);
    if( !pCodecPCM ){
        fprintf(stderr, "PCM codec not found\n");
        exit(1);
    }
    pCodecCtxPCM = avcodec_alloc_context();

	pCodecCtxPCM->bit_rate = 1536000;
	pCodecCtxPCM->channels = 2;
	pCodecCtxPCM->sample_rate = 48000;

    /* open it */
    if( avcodec_open(pCodecCtxPCM, pCodecPCM) < 0 ){
        fprintf(stderr, "could not open PCM codec\n");
        exit(1);
    }
	pCodecCtxPCM->sample_fmt = pCodecCtxPCM->codec->sample_fmts[0];

    // Open input file
	i = av_open_input_file(&pFormatCtx, filename, NULL, 0, NULL); 
	if( i != 0 ){
		fprintf(stderr, "Couldn't open file '%s'. Error code = %i\n", filename, i);    
	}

    // Retrieve stream information
	if(av_find_stream_info(pFormatCtx)<0){
		fprintf(stderr, "Couldn't find stream information.");    
        return -1;
	}

    // Dump information about file onto standard error
    dump_format(pFormatCtx, 0, filename, 0); 

	// Find the first audio stream
    audioStream=-1;
    for(i=0; i<pFormatCtx->nb_streams; i++)
        if(pFormatCtx->streams[i]->codec->codec_type==CODEC_TYPE_AUDIO)
        {
            audioStream=i;
            break;
        }
    if(audioStream==-1)
        return -1; // Didn't find a stream

	printf("Found audioStream id = %d\n", audioStream );

    // Get a pointer to the codec context for the video stream
    pCodecCtx = pFormatCtx->streams[audioStream]->codec;

    // Find the decoder for the video stream
    pCodec=avcodec_find_decoder(pCodecCtx->codec_id);
    if(pCodec==NULL)
        return -1; // Codec not found

	// Open codec
    if(avcodec_open(pCodecCtx, pCodec)<0)
        return -1; // Could not open codec

    // enable stereo downmix
    pCodecCtx->request_channels = 2;

    // Read frames and save first five frames to disk
    i=0;
	sample_count = 0;
    while(av_read_frame(pFormatCtx, &packet)>=0)
    {
		printf("packet.stream_index=%i\n", packet.stream_index );
		printf("packet.pts=0x%08X\n", (int)packet.pts);
		printf("packet.dts=0x%08X\n", (int)packet.dts);
		printf("packet.size=0x%08X\n", (int)packet.size);
		printf("packet.stream_index=0x%08X\n", (int)packet.stream_index);
		printf("packet.flags=0x%08X\n", (int)packet.flags);
		printf("packet.duration=0x%08X\n", (int)packet.duration);
		printf("packet.priv=0x%08X\n", (int)packet.priv);
		printf("packet.pos=0x%08X\n", (int)packet.pos);

		// Is this a packet from the video stream?
        if(packet.stream_index==audioStream)
        {
			printf("packet.data = 0x%08X, packet.size=0x%04X\n", 
				(int)packet.data, packet.size );

			fwrite( packet.data, 1, packet.size, outfile );

			if( packet.data )
				dump_hex( packet.data, packet.size );

			out_size = AVCODEC_MAX_AUDIO_FRAME_SIZE;
			len = avcodec_decode_audio3(pCodecCtx, samples, &out_size, &packet);

			printf("\nnumber of processed input bytes = %i\n", len );
			printf("number of decoded bytes = %i\n", out_size );
			sample_count += out_size;
			printf("overall sample count = %i\n", sample_count );
			if (len < 0) {
				fprintf(stderr, "Error while decoding\n");
	            exit(1);
		    }
	        if (len != packet.size) {
		        fprintf(stderr, "Not entire frame / buffer was decoded (%i of %i)\n", len, packet.size);
			    // exit(1);
			}
			printf("\nraw samples:\n");
	        if (out_size > 0) {
				dump_hex( (unsigned char *)samples, out_size );
			}

			len = avcodec_encode_audio(pCodecCtxPCM, pcmbuf, out_size, samples);
			if (len < 0) {
				fprintf(stderr, "Error while encoding\n");
		    }

			printf("encoded audio (len = %i):\n", len);
	        if( len > 0 ){
				dump_hex( (unsigned char *)pcmbuf, len );
			}

		}
		// Free the packet that was allocated by av_read_frame
		av_free_packet(&packet);
    }
 
	fclose( outfile );
    avcodec_close(pCodecCtxPCM);
    av_free(pCodecCtxPCM);
    avcodec_close(pCodecCtx);
    av_free(pCodecCtx);

	return 0;
}

int main(int argc, char **argv)
{
	clock_t start_clock;

    if( argc !=3 ){
        fprintf(stderr, "Usage: %s input output.pcm\n", argv[0]);
        exit(1);
    }

    /* must be called before using avcodec lib */
    avcodec_init();

    /* register all the codecs */
    avcodec_register_all();

	/* Register all formats and codecs */
    av_register_all(); 

	start_clock = clock();
	demux_audio_example( argv[1], argv[2] );
	fprintf(stderr, "Elapsed time = %li ticks.\n", (clock() - start_clock) );
	
	return 0;
}
